declare class Console {
	trace(...args: any): void;
	debug(...args: any): void;
	info(...args: any): void;
	warn(...args: any): void;
	error(...args: any): void;
	log(...args: any): void;
}
type Listener = (...args: any[]) => void;
interface EventMap {
	error: (error: Error) => void;
	newListener: (event: string | number | symbol, listener: Listener) => void;
	removeListener: (event: string | number | symbol, listener: Listener) => void;
}
interface AnyEmmiter extends EventMap {
	[key: symbol]: Listener;
	[key: string]: Listener;
	[key: number]: Listener;
}
declare class EventEmitter<Events extends EventMap = AnyEmmiter> {
	static listenerCount(emitter: EventEmitter, type: string | number): number;
	static defaultMaxListeners: number;
	addListener<E extends keyof Events>(event: E, listener: Events[E]): this;
	on<E extends keyof Events>(event: E, listener: Events[E]): this;
	once<E extends keyof Events>(event: E, listener: Events[E]): this;
	prependListener<E extends keyof Events>(event: E, listener: Events[E]): this;
	prependOnceListener<E extends keyof Events>(event: E, listener: Events[E]): this;
	off<E extends keyof Events>(event: E, listener: Events[E]): this;
	removeAllListeners<E extends keyof Events>(event?: E): this;
	removeListener<E extends keyof Events>(event: E, listener: Events[E]): this;
	emit<E extends keyof Events>(event: E, ...args: Parameters<(Events & AnyEmmiter)[E]>): boolean;
	// The sloppy `eventNames()` return type is to mitigate type incompatibilities - see #5
	eventNames(): (string | symbol)[];
	rawListeners<E extends keyof Events>(event: E): Events[E][];
	listeners<E extends keyof Events>(event: E): Events[E][];
	listenerCount<E extends keyof Events>(event: E): number;
	getMaxListeners(): number;
	setMaxListeners(maxListeners: number): this;
}
type UnitEventType = keyof typeof stringToHandle;
declare const stringToHandle: {
	damaged: HUnitEvent;
	damaging: HUnitEvent;
	selected: HUnitEvent;
	deselected: HUnitEvent;
	death: HUnitEvent;
	decay: HUnitEvent;
	detected: HUnitEvent;
	hidden: HUnitEvent;
	stateLimit: HUnitEvent;
	acquiredTarget: HUnitEvent;
	targetInRange: HUnitEvent;
	attacked: HUnitEvent;
	rescued: HUnitEvent;
	constructCancel: HUnitEvent;
	constructFinish: HUnitEvent;
	upgradeStart: HUnitEvent;
	upgradeCancel: HUnitEvent;
	upgradeFinish: HUnitEvent;
	trainStart: HUnitEvent;
	trainCancel: HUnitEvent;
	trainFinish: HUnitEvent;
	researchStart: HUnitEvent;
	researchCancel: HUnitEvent;
	researchFinish: HUnitEvent;
	issuedOrder: HUnitEvent;
	issuedPointOrder: HUnitEvent;
	issuedTargetOrder: HUnitEvent;
	heroLevel: HUnitEvent;
	heroSkill: HUnitEvent;
	heroRevivable: HUnitEvent;
	heroReviveStart: HUnitEvent;
	heroReviveCancel: HUnitEvent;
	heroReviveFinish: HUnitEvent;
	summon: HUnitEvent;
	dropItem: HUnitEvent;
	pickupItem: HUnitEvent;
	useItem: HUnitEvent;
	loaded: HUnitEvent;
	attackFinished: HUnitEvent;
	decayFinished: HUnitEvent;
};
declare class TriggerEvent<E extends string = string> {
	readonly event: E;
	constructor(event: E);
}
declare class TriggerUnitEvent<E extends UnitEventType = UnitEventType> extends TriggerEvent {
	readonly triggerUnit: Unit;
	constructor(event: E);
}
declare class UnitEventDeath extends TriggerUnitEvent<"death"> {
	readonly killingUnit: Unit;
	constructor();
}
declare class UnitEventDetected extends TriggerUnitEvent<"detected"> {
	readonly detectingPlayer: Player;
	constructor();
}
declare class UnitEventAcquiredTarget extends TriggerUnitEvent<"acquiredTarget"> {
	readonly target: Unit;
	constructor();
}
declare class UnitEventTargetInRange extends TriggerUnitEvent<"targetInRange"> {
	readonly target: Unit;
	constructor();
}
declare class UnitEventAttacked extends TriggerUnitEvent<"attacked"> {
	readonly attacker: Unit;
	constructor();
}
declare class UnitEventRescued extends TriggerUnitEvent<"rescued"> {
	readonly rescuer: Unit;
	constructor();
}
declare class UnitEventConstructFinish extends TriggerUnitEvent<"constructFinish"> {
	readonly constructedStructure: Unit;
	constructor();
}
declare class UnitEventTrainStart extends TriggerUnitEvent<"trainStart"> {
	readonly trainedUnitType: number;
	constructor();
}
declare class UnitEventTrainCancel extends TriggerUnitEvent<"trainCancel"> {
	readonly trainedUnitType: number;
	constructor();
}
declare class UnitEventTrainFinish extends TriggerUnitEvent<"trainFinish"> {
	readonly trainedUnit: Unit;
	constructor();
}
declare class UnitEventResearchStart extends TriggerUnitEvent<"researchStart"> {
	readonly researched: number;
	constructor();
}
declare class UnitEventResearchCancel extends TriggerUnitEvent<"researchCancel"> {
	readonly researched: number;
	constructor();
}
declare class UnitEventResearchFinish extends TriggerUnitEvent<"researchFinish"> {
	readonly researched: number;
	constructor();
}
declare class UnitEventIssuedOrder extends TriggerUnitEvent<"issuedOrder"> {
	readonly issuedOrderId: number;
	constructor();
}
declare class UnitEventIssuedPointOrder extends TriggerUnitEvent<"issuedPointOrder"> {
	readonly issuedOrderId: number;
	readonly orderPointX: number;
	readonly orderPointY: number;
	constructor();
}
declare class UnitEventIssuedTargetOrder extends TriggerUnitEvent<"issuedTargetOrder"> {
	readonly issuedOrderId: number;
	readonly orderTarget: Widget;
	constructor();
}
declare class UnitEventHeroSkill extends TriggerUnitEvent<"heroSkill"> {
	readonly learnedSkill: number;
	readonly learnedSkillLevel: number;
	constructor();
}
declare class UnitEventHeroRevivable extends TriggerUnitEvent<"heroRevivable"> {
	readonly revivableUnit: Unit;
	constructor();
}
declare class UnitEventHeroReviveStart extends TriggerUnitEvent<"heroReviveStart"> {
	readonly revivingUnit: Unit;
	constructor();
}
declare class UnitEventHeroReviveCancel extends TriggerUnitEvent<"heroReviveCancel"> {
	readonly revivingUnit: Unit;
	constructor();
}
declare class UnitEventHeroReviveFinish extends TriggerUnitEvent<"heroReviveFinish"> {
	readonly revivingUnit: Unit;
	constructor();
}
declare class UnitEventSummon extends TriggerUnitEvent<"summon"> {
	readonly summonedUnit: Unit;
	constructor();
}
declare class UnitEventDropItem extends TriggerUnitEvent<"dropItem"> {
	readonly droppedItem: Item;
	constructor();
}
declare class UnitEventPickupItem extends TriggerUnitEvent<"pickupItem"> {
	readonly pickedupItem: Item;
	constructor();
}
declare class UnitEventUseItem extends TriggerUnitEvent<"useItem"> {
	readonly usedItem: Item;
	constructor();
}
declare class UnitEventLoaded extends TriggerUnitEvent<"loaded"> {
	readonly transport: Unit;
	constructor();
}
declare class UnitEventStateLimit extends TriggerUnitEvent<"stateLimit"> {
	readonly eventUnitState: HUnitState;
	constructor();
}
interface IDisposable {
	dispose: () => void;
}
declare class UnitStateEmiter implements IDisposable {
	#private;
	constructor(target: Unit, unitState: UnitState, limitOp: LimitOp, value: number);
	get unitState(): UnitState;
	get limitOp(): LimitOp;
	get value(): number;
	get emitSymbol(): StateEventSymbol;
	dispose(): void;
}
declare const stateEvent: unique symbol;
declare global {
	/**
	 * A low-level object that replaces the handle type in war3js.
	 * It is not recommended to use it as an object, as some fields may start to be used by the war3js backend.
	 */
	interface HandleHolder<T extends string = string, P = unknown> {
		/**
		 * Returns jass handle type. For fake handles this is _enum.
		 */
		get type(): T;
		/**
		 * The library object this handle is bound to
		 */
		payload: P;
		/**
		 * Compares handle's internal pointers as numbers. ``null`` is interpreted as 0
		 * @param handle another handle for compare
		 * @returns true if jass
		 */
		equals: (handle: HandleHolder<string> | null) => boolean;
	}

    export type HandleConstructor<T extends string = string, H extends Handle = Handle> = new (handle: HandleHolder<T>) => H;

    interface HandleEventMap extends EventMap {
        error: (error: Error) => void;
        remove: () => void;
    }

    class Handle<T extends HandleEventMap = HandleEventMap> extends EventEmitter<T> {
        #private;
        constructor(handleHolder: Handle | HandleHolder<string>);
        get handle(): HandleHolder<string, unknown>;
        equals(anotherHandle: Handle): boolean;
    }

    interface WidgetEventMap extends HandleEventMap {
        death: (event: any) => void;
        damaging: () => void;
        damaged: () => void;
    }

    class Widget<T extends WidgetEventMap = WidgetEventMap> extends Handle<T> {
        constructor(handleHolder: HandleHolder<"widget" | "unit" | "item" | "destructable"> | Widget);
        /**
         *             Class chain property alias for {@link natives.GetWidgetLife | GetWidgetLife} and {@link natives.SetWidgetLife | SetWidgetLife}
         *             
         */
        public life(): number;
        public life<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetWidgetX | GetWidgetX} and {@link natives.SetWidgetX | SetWidgetX}
         *             
         */
        public x(): number;
        public x<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetWidgetY | GetWidgetY} and {@link natives.SetWidgetY | SetWidgetY}
         *             
         */
        public y(): number;
        public y<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.IsWidgetVisible | IsWidgetVisible} and {@link natives.SetWidgetVisible | SetWidgetVisible}
         *             
         */
        public visible(): boolean;
        public visible<T = boolean>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.IsWidgetInvulnerable | IsWidgetInvulnerable} and {@link natives.SetWidgetInvulnerable | SetWidgetInvulnerable}
         *             
         */
        public invulnerable(): boolean;
        public invulnerable<T = boolean>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetWidgetPositionLoc | GetWidgetPositionLoc} and {@link natives.SetWidgetPositionLoc | SetWidgetPositionLoc}
         *             
         */
        public positionLoc(): HLocation;
        public positionLoc<T = HLocation>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetWidgetTimeScale | GetWidgetTimeScale} and {@link natives.SetWidgetTimeScale | SetWidgetTimeScale}
         *             
         */
        public timeScale(): number;
        public timeScale<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetWidgetScale | GetWidgetScale} and {@link natives.SetWidgetTimeScale | SetWidgetTimeScale}
         *             
         */
        public scale(): number;
        public scale<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetWidgetYaw | GetWidgetYaw} and {@link natives.SetWidgetYaw | SetWidgetYaw}
         *             
         */
        public yaw(): number;
        public yaw<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetWidgetPitch | GetWidgetPitch} and {@link natives.SetWidgetPitch | SetWidgetPitch}
         *             
         */
        public pitch(): number;
        public pitch<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetWidgetRoll | GetWidgetRoll} and {@link natives.SetWidgetRoll | SetWidgetRoll}
         *             
         */
        public roll(): number;
        public roll<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetWidgetModel | GetWidgetModel} and {@link natives.SetWidgetModel | SetWidgetModel}
         *             
         */
        public model(): string;
        public model<T = string>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetWidgetAnimationOffsetPercent | GetWidgetAnimationOffsetPercent} and {@link natives.SetWidgetAnimationOffsetPercent | SetWidgetAnimationOffsetPercent}
         *             
         */
        public animationOffsetPercent(): number;
        public animationOffsetPercent<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class member alias for {@link natives.AddIndicator | AddIndicator}
         *             
         */
        public addIndicator(red: number, green: number, blue: number, alpha: number): void;
        /**
         *             Class member alias for {@link natives.GetWidgetSprite | GetWidgetSprite}
         *             
         */
        public getSprite(): HandleHolder<"sprite">;
        /**
         *             Class member alias for {@link natives.GetWidgetTypeId | GetWidgetTypeId}
         *             
         */
        public getTypeId(): number;
        /**
         *             Class member alias for {@link natives.IsWidgetTargetAllowed | IsWidgetTargetAllowed}
         *             
         */
        public isTargetAllowed(target: Widget, whichFlags: HandleHolder<"targetflag">): boolean;
        /**
         *             Class member alias for {@link natives.SetWidgetPosition | SetWidgetPosition}
         *             
         */
        public setPosition(x: number, y: number): void;
        /**
         *             Class member alias for {@link natives.GetWidgetScreenX | GetWidgetScreenX}
         *             
         */
        public getScreenX(): number;
        /**
         *             Class member alias for {@link natives.GetWidgetScreenY | GetWidgetScreenY}
         *             
         */
        public getScreenY(): number;
        /**
         *             Class member alias for {@link natives.GetWidgetVertexColour | GetWidgetVertexColour}
         *             
         */
        public getVertexColour(): number;
        /**
         *             Class member alias for {@link natives.SetWidgetVertexColour | SetWidgetVertexColour}
         *             
         */
        public setVertexColour(red: number, green: number, blue: number, alpha: number): void;
        /**
         *             Class member alias for {@link natives.GetWidgetFacing | GetWidgetFacing}
         *             
         */
        public getFacing(): number;
        /**
         *             Class member alias for {@link natives.SetWidgetFacing | SetWidgetFacing}
         *             
         */
        public setFacing(facing: number, isInstant: boolean): void;
        /**
         *             Class member alias for {@link natives.SetWidgetMatrixScale | SetWidgetMatrixScale}
         *             
         */
        public setMatrixScale(x: number, y: number, z: number): void;
        /**
         *             Class member alias for {@link natives.ResetWidgetMatrix | ResetWidgetMatrix}
         *             
         */
        public resetMatrix(): void;
        /**
         *             Class member alias for {@link natives.SetWidgetOrientationEx | SetWidgetOrientationEx}
         *             
         */
        public setOrientationEx(yaw: number, pitch: number, roll: number, eulerOrder: number): void;
        /**
         *             Class member alias for {@link natives.SetWidgetOrientation | SetWidgetOrientation}
         *             
         */
        public setOrientation(yaw: number, pitch: number, roll: number): void;
        /**
         *             Class member alias for {@link natives.SetWidgetModelEx | SetWidgetModelEx}
         *             
         */
        public setModelEx(modelFile: string, playerId: number): void;
        /**
         *             Class member alias for {@link natives.SetWidgetMaterialTexture | SetWidgetMaterialTexture}
         *             
         */
        public setMaterialTexture(textureName: string, materialId: number, textureIndex: number): void;
        /**
         *             Class member alias for {@link natives.SetWidgetTexture | SetWidgetTexture}
         *             
         */
        public setTexture(textureName: string, textureIndex: number): void;
        /**
         *             Class member alias for {@link natives.SetWidgetReplaceableTexture | SetWidgetReplaceableTexture}
         *             
         */
        public setReplaceableTexture(textureName: string, textureIndex: number): void;
        /**
         *             Class member alias for {@link natives.GetWidgetModelObjectX | GetWidgetModelObjectX}
         *             
         */
        public getModelObjectX(whichObject: string): number;
        /**
         *             Class member alias for {@link natives.GetWidgetModelObjectY | GetWidgetModelObjectY}
         *             
         */
        public getModelObjectY(whichObject: string): number;
        /**
         *             Class member alias for {@link natives.GetWidgetModelObjectZ | GetWidgetModelObjectZ}
         *             
         */
        public getModelObjectZ(whichObject: string): number;
        /**
         *             Class member alias for {@link natives.GetWidgetModelObjectPositionLoc | GetWidgetModelObjectPositionLoc}
         *             
         */
        public getModelObjectPositionLoc(whichObject: string): HLocation;
        /**
         *             Class member alias for {@link natives.GetWidgetCurrentAnimationId | GetWidgetCurrentAnimationId}
         *             
         */
        public getCurrentAnimationId(): number;
        /**
         *             Class member alias for {@link natives.GetWidgetCurrentAnimationName | GetWidgetCurrentAnimationName}
         *             
         */
        public getCurrentAnimationName(): string;
        /**
         *             Class member alias for {@link natives.SetWidgetAnimationWithRarityByIndex | SetWidgetAnimationWithRarityByIndex}
         *             
         */
        public setAnimationWithRarityByIndex(animIndex: number, rarity: HandleHolder<"raritycontrol">): void;
        /**
         *             Class member alias for {@link natives.SetWidgetAnimationWithRarity | SetWidgetAnimationWithRarity}
         *             
         */
        public setAnimationWithRarity(animationName: string, rarity: HandleHolder<"raritycontrol">): void;
        /**
         *             Class member alias for {@link natives.QueueWidgetAnimationByIndex | QueueWidgetAnimationByIndex}
         *             
         */
        public queueAnimationByIndex(animIndex: number): void;
        /**
         *             Class member alias for {@link natives.QueueWidgetAnimation | QueueWidgetAnimation}
         *             
         */
        public queueAnimation(animationName: string): void;
    }

    interface PlayerEventMap extends HandleEventMap {
    }

    interface Player {
    }

    class Player<T extends PlayerEventMap = PlayerEventMap> extends Handle<T> {
        constructor(handleHolder: HandleHolder<"player">);
        static getById<T extends Player = Player>(slot: number): T;
        static localPlayer<T extends Player = Player>(): T;
        /**
         *             Class chain property alias for {@link natives.GetPlayerTeam | GetPlayerTeam} and {@link natives.SetPlayerTeam | SetPlayerTeam}
         *             
         */
        public team(): number;
        public team<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetPlayerStartLocation | GetPlayerStartLocation} and {@link natives.SetPlayerStartLocation | SetPlayerStartLocation}
         *             
         */
        public startLocation(): number;
        public startLocation<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetPlayerColor | GetPlayerColor} and {@link natives.SetPlayerColor | SetPlayerColor}
         *             
         */
        public color(): HandleHolder<"playercolor">;
        public color<T = HandleHolder<"playercolor">>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetPlayerSelectable | GetPlayerSelectable} and {@link natives.SetPlayerRaceSelectable | SetPlayerRaceSelectable}
         *             
         */
        public selectable(): boolean;
        public selectable<T = boolean>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetPlayerController | GetPlayerController} and {@link natives.SetPlayerController | SetPlayerController}
         *             
         */
        public controller(): HandleHolder<"mapcontrol">;
        public controller<T = HandleHolder<"mapcontrol">>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetPlayerName | GetPlayerName} and {@link natives.SetPlayerName | SetPlayerName}
         *             
         */
        public name(): string;
        public name<T = string>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetPlayerHandicap | GetPlayerHandicap} and {@link natives.SetPlayerHandicap | SetPlayerHandicap}
         *             
         */
        public handicap(): number;
        public handicap<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetPlayerHandicapXP | GetPlayerHandicapXP} and {@link natives.SetPlayerHandicapXP | SetPlayerHandicapXP}
         *             
         */
        public handicapXP(): number;
        public handicapXP<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetPlayerHandicapReviveTime | GetPlayerHandicapReviveTime} and {@link natives.SetPlayerHandicapReviveTime | SetPlayerHandicapReviveTime}
         *             
         */
        public handicapReviveTime(): number;
        public handicapReviveTime<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetPlayerHandicapDamage | GetPlayerHandicapDamage} and {@link natives.SetPlayerHandicapDamage | SetPlayerHandicapDamage}
         *             
         */
        public handicapDamage(): number;
        public handicapDamage<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class member alias for {@link natives.ForcePlayerStartLocation | ForcePlayerStartLocation}
         *             
         */
        public forceStartLocation(startLocIndex: number): void;
        /**
         *             Class member alias for {@link natives.SetPlayerAlliance | SetPlayerAlliance}
         *             
         */
        public setAlliance(otherPlayer: Player, whichAllianceSetting: HandleHolder<"alliancetype">, value: boolean): void;
        /**
         *             Class member alias for {@link natives.SetPlayerTaxRate | SetPlayerTaxRate}
         *             
         */
        public setTaxRate(otherPlayer: Player, whichResource: HandleHolder<"playerstate">, rate: number): void;
        /**
         *             Class member alias for {@link natives.GetPlayerSlotState | GetPlayerSlotState}
         *             
         */
        public getSlotState(): HandleHolder<"playerslotstate">;
        /**
         *             Class member alias for {@link natives.GetPlayerTaxRate | GetPlayerTaxRate}
         *             
         */
        public getTaxRate(otherPlayer: Player, whichResource: HandleHolder<"playerstate">): number;
        /**
         *             Class member alias for {@link natives.IsPlayerRacePrefSet | IsPlayerRacePrefSet}
         *             
         */
        public isRacePrefSet(pref: HandleHolder<"racepreference">): boolean;
        /**
         *             Class member alias for {@link natives.GetTournamentScore | GetTournamentScore}
         *             
         */
        public getTournamentScore(): number;
        /**
         *             Class member alias for {@link natives.CreateUnit | CreateUnit}
         *             
         */
        public createUnit(unitid: number, x: number, y: number, face: number): Unit;
        /**
         *             Class member alias for {@link natives.CreateUnitByName | CreateUnitByName}
         *             
         */
        public createUnitByName(unitname: string, x: number, y: number, face: number): Unit;
        /**
         *             Class member alias for {@link natives.CreateUnitAtLoc | CreateUnitAtLoc}
         *             
         */
        public createUnitAtLoc(unitid: number, whichLocation: HLocation, face: number): Unit;
        /**
         *             Class member alias for {@link natives.CreateUnitAtLocByName | CreateUnitAtLocByName}
         *             
         */
        public createUnitAtLocByName(unitname: string, whichLocation: HLocation, face: number): Unit;
        /**
         *             Class member alias for {@link natives.CreateCorpse | CreateCorpse}
         *             
         */
        public createCorpse(unitid: number, x: number, y: number, face: number): Unit;
        /**
         *             Class member alias for {@link natives.IssueNeutralImmediateOrder | IssueNeutralImmediateOrder}
         *             
         */
        public issueNeutralImmediateOrder(neutralStructure: Unit, unitToBuild: string): boolean;
        /**
         *             Class member alias for {@link natives.IssueNeutralImmediateOrderById | IssueNeutralImmediateOrderById}
         *             
         */
        public issueNeutralImmediateOrderById(neutralStructure: Unit, unitId: number): boolean;
        /**
         *             Class member alias for {@link natives.IssueNeutralPointOrder | IssueNeutralPointOrder}
         *             
         */
        public issueNeutralPointOrder(neutralStructure: Unit, unitToBuild: string, x: number, y: number): boolean;
        /**
         *             Class member alias for {@link natives.IssueNeutralPointOrderById | IssueNeutralPointOrderById}
         *             
         */
        public issueNeutralPointOrderById(neutralStructure: Unit, unitId: number, x: number, y: number): boolean;
        /**
         *             Class member alias for {@link natives.IssueNeutralTargetOrder | IssueNeutralTargetOrder}
         *             
         */
        public issueNeutralTargetOrder(neutralStructure: Unit, unitToBuild: string, target: Widget): boolean;
        /**
         *             Class member alias for {@link natives.IssueNeutralTargetOrderById | IssueNeutralTargetOrderById}
         *             
         */
        public issueNeutralTargetOrderById(neutralStructure: Unit, unitId: number, target: Widget): boolean;
        /**
         *             Class member alias for {@link natives.IsPlayerAlly | IsPlayerAlly}
         *             
         */
        public isAlly(otherPlayer: Player): boolean;
        /**
         *             Class member alias for {@link natives.IsPlayerEnemy | IsPlayerEnemy}
         *             
         */
        public isEnemy(otherPlayer: Player): boolean;
        /**
         *             Class member alias for {@link natives.IsPlayerInForce | IsPlayerInForce}
         *             
         */
        public isInForce(whichForce: HandleHolder<"force">): boolean;
        /**
         *             Class member alias for {@link natives.IsPlayerObserver | IsPlayerObserver}
         *             
         */
        public isObserver(): boolean;
        /**
         *             Class member alias for {@link natives.GetPlayerRace | GetPlayerRace}
         *             
         */
        public getRace(): HandleHolder<"race">;
        /**
         *             Class member alias for {@link natives.GetPlayerId | GetPlayerId}
         *             
         */
        public getId(): number;
        /**
         *             Class member alias for {@link natives.GetPlayerUnitCount | GetPlayerUnitCount}
         *             
         */
        public getUnitCount(includeIncomplete: boolean): number;
        /**
         *             Class member alias for {@link natives.GetPlayerTypedUnitCount | GetPlayerTypedUnitCount}
         *             
         */
        public getTypedUnitCount(unitName: string, includeIncomplete: boolean, includeUpgrades: boolean): number;
        /**
         *             Class member alias for {@link natives.GetPlayerStructureCount | GetPlayerStructureCount}
         *             
         */
        public getStructureCount(includeIncomplete: boolean): number;
        /**
         *             Class member alias for {@link natives.GetPlayerState | GetPlayerState}
         *             
         */
        public getState(whichPlayerState: HandleHolder<"playerstate">): number;
        /**
         *             Class member alias for {@link natives.GetPlayerScore | GetPlayerScore}
         *             
         */
        public getScore(whichPlayerScore: HandleHolder<"playerscore">): number;
        /**
         *             Class member alias for {@link natives.GetPlayerAlliance | GetPlayerAlliance}
         *             
         */
        public getAlliance(otherPlayer: Player, whichAllianceSetting: HandleHolder<"alliancetype">): boolean;
        /**
         *             Class member alias for {@link natives.SetPlayerTechMaxAllowed | SetPlayerTechMaxAllowed}
         *             
         */
        public setTechMaxAllowed(techid: number, maximum: number): void;
        /**
         *             Class member alias for {@link natives.GetPlayerTechMaxAllowed | GetPlayerTechMaxAllowed}
         *             
         */
        public getTechMaxAllowed(techid: number): number;
        /**
         *             Class member alias for {@link natives.AddPlayerTechResearched | AddPlayerTechResearched}
         *             
         */
        public addTechResearched(techid: number, levels: number): void;
        /**
         *             Class member alias for {@link natives.SetPlayerTechResearched | SetPlayerTechResearched}
         *             
         */
        public setTechResearched(techid: number, setToLevel: number): void;
        /**
         *             Class member alias for {@link natives.GetPlayerTechResearched | GetPlayerTechResearched}
         *             
         */
        public getTechResearched(techid: number, specificonly: boolean): boolean;
        /**
         *             Class member alias for {@link natives.GetPlayerTechCount | GetPlayerTechCount}
         *             
         */
        public getTechCount(techid: number, specificonly: boolean): number;
        /**
         *             Class member alias for {@link natives.CripplePlayer | CripplePlayer}
         *             
         */
        public cripple(toWhichPlayers: HandleHolder<"force">, flag: boolean): void;
        /**
         *             Class member alias for {@link natives.SetPlayerAbilityAvailable | SetPlayerAbilityAvailable}
         *             
         */
        public setAbilityAvailable(abilid: number, avail: boolean): void;
        /**
         *             Class member alias for {@link natives.SetPlayerState | SetPlayerState}
         *             
         */
        public setState(whichPlayerState: HandleHolder<"playerstate">, value: number): void;
        /**
         *             Class member alias for {@link natives.RemovePlayer | RemovePlayer}
         *             
         */
        public remove(gameResult: HandleHolder<"playergameresult">): void;
        /**
         *             Class member alias for {@link natives.CachePlayerHeroData | CachePlayerHeroData}
         *             
         */
        public cacheHeroData(): void;
        /**
         *             Class member alias for {@link natives.SetFogStateRect | SetFogStateRect}
         *             
         */
        public setFogStateRect(whichState: HandleHolder<"fogstate">, where: HandleHolder<"rect">, useSharedVision: boolean): void;
        /**
         *             Class member alias for {@link natives.SetFogStateRadius | SetFogStateRadius}
         *             
         */
        public setFogStateRadius(whichState: HandleHolder<"fogstate">, centerx: number, centerY: number, radius: number, useSharedVision: boolean): void;
        /**
         *             Class member alias for {@link natives.SetFogStateRadiusLoc | SetFogStateRadiusLoc}
         *             
         */
        public setFogStateRadiusLoc(whichState: HandleHolder<"fogstate">, center: HLocation, radius: number, useSharedVision: boolean): void;
        /**
         *             Class member alias for {@link natives.CreateFogModifierRect | CreateFogModifierRect}
         *             
         */
        public createFogModifierRect(whichState: HandleHolder<"fogstate">, where: HandleHolder<"rect">, useSharedVision: boolean, afterUnits: boolean): HandleHolder<"fogmodifier">;
        /**
         *             Class member alias for {@link natives.CreateFogModifierRadius | CreateFogModifierRadius}
         *             
         */
        public createFogModifierRadius(whichState: HandleHolder<"fogstate">, centerx: number, centerY: number, radius: number, useSharedVision: boolean, afterUnits: boolean): HandleHolder<"fogmodifier">;
        /**
         *             Class member alias for {@link natives.CreateFogModifierRadiusLoc | CreateFogModifierRadiusLoc}
         *             
         */
        public createFogModifierRadiusLoc(whichState: HandleHolder<"fogstate">, center: HLocation, radius: number, useSharedVision: boolean, afterUnits: boolean): HandleHolder<"fogmodifier">;
        /**
         *             Class member alias for {@link natives.DialogDisplay | DialogDisplay}
         *             
         */
        public dialogDisplay(whichDialog: HandleHolder<"dialog">, flag: boolean): void;
        /**
         *             Class member alias for {@link natives.DisplayTextToPlayer | DisplayTextToPlayer}
         *             
         */
        public displayTextTo(x: number, y: number, message: string): void;
        /**
         *             Class member alias for {@link natives.DisplayTimedTextToPlayer | DisplayTimedTextToPlayer}
         *             
         */
        public displayTimedTextTo(x: number, y: number, duration: number, message: string): void;
        /**
         *             Class member alias for {@link natives.DisplayTimedTextFromPlayer | DisplayTimedTextFromPlayer}
         *             
         */
        public displayTimedTextFrom(x: number, y: number, duration: number, message: string): void;
        /**
         *             Class member alias for {@link natives.PlayerSetLeaderboard | PlayerSetLeaderboard}
         *             
         */
        public setLeaderboard(lb: HandleHolder<"leaderboard">): void;
        /**
         *             Class member alias for {@link natives.PlayerGetLeaderboard | PlayerGetLeaderboard}
         *             
         */
        public getLeaderboard(): HandleHolder<"leaderboard">;
        /**
         *             Class member alias for {@link natives.SetBlight | SetBlight}
         *             
         */
        public setBlight(x: number, y: number, radius: number, addBlight: boolean): void;
        /**
         *             Class member alias for {@link natives.SetBlightRect | SetBlightRect}
         *             
         */
        public setBlightRect(r: HandleHolder<"rect">, addBlight: boolean): void;
        /**
         *             Class member alias for {@link natives.SetBlightPoint | SetBlightPoint}
         *             
         */
        public setBlightPoint(x: number, y: number, addBlight: boolean): void;
        /**
         *             Class member alias for {@link natives.SetBlightLoc | SetBlightLoc}
         *             
         */
        public setBlightLoc(whichLocation: HLocation, radius: number, addBlight: boolean): void;
        /**
         *             Class member alias for {@link natives.CreateBlightedGoldmine | CreateBlightedGoldmine}
         *             
         */
        public createBlightedGoldmine(x: number, y: number, face: number): Unit;
        /**
         *             Class member alias for {@link natives.StartMeleeAI | StartMeleeAI}
         *             
         */
        public startMeleeAI(script: string): void;
        /**
         *             Class member alias for {@link natives.StartCampaignAI | StartCampaignAI}
         *             
         */
        public startCampaignAI(script: string): void;
        /**
         *             Class member alias for {@link natives.CommandAI | CommandAI}
         *             
         */
        public commandAI(command: number, data: number): void;
        /**
         *             Class member alias for {@link natives.PauseCompAI | PauseCompAI}
         *             
         */
        public pauseCompAI(pause: boolean): void;
        /**
         *             Class member alias for {@link natives.GetAIDifficulty | GetAIDifficulty}
         *             
         */
        public getAIDifficulty(): HandleHolder<"aidifficulty">;
        /**
         *             Class member alias for {@link natives.RemoveAllGuardPositions | RemoveAllGuardPositions}
         *             
         */
        public removeAllGuardPositions(): void;
        /**
         *             Class member alias for {@link natives.DisplayWarningMessage | DisplayWarningMessage}
         *             
         */
        public displayWarningMessage(message: string): void;
        /**
         *             Class member alias for {@link natives.DisplayTimedWarningMessage | DisplayTimedWarningMessage}
         *             
         */
        public displayTimedWarningMessage(duration: number, message: string): void;
        /**
         *             Class member alias for {@link natives.DisplayChatMessage | DisplayChatMessage}
         *             
         */
        public displayChatMessage(recipient: number, message: string): void;
        /**
         *             Class member alias for {@link natives.DisplayTimedChatMessage | DisplayTimedChatMessage}
         *             
         */
        public displayTimedChatMessage(recipient: number, duration: number, message: string): void;
        /**
         *             Class member alias for {@link natives.DisplayTopMessage | DisplayTopMessage}
         *             
         */
        public displayTopMessage(message: string): void;
        /**
         *             Class member alias for {@link natives.DisplayTimedTopMessage | DisplayTimedTopMessage}
         *             
         */
        public displayTimedTopMessage(duration: number, message: string): void;
        /**
         *             Class member alias for {@link natives.GetUnitSelectedCountByPlayer | GetUnitSelectedCountByPlayer}
         *             
         */
        public getUnitSelectedCountBy(): number;
        /**
         *             Class member alias for {@link natives.GetUnitSelected | GetUnitSelected}
         *             
         */
        public getUnitSelected(): Unit;
        /**
         *             Class member alias for {@link natives.GetUnitInSelectionByIndex | GetUnitInSelectionByIndex}
         *             
         */
        public getUnitInSelectionByIndex(index: number): Unit;
        /**
         *             Class member alias for {@link natives.GetFirstUnitInSelection | GetFirstUnitInSelection}
         *             
         */
        public getFirstUnitInSelection(): Unit;
        /**
         *             Class member alias for {@link natives.GetLastUnitInSelection | GetLastUnitInSelection}
         *             
         */
        public getLastUnitInSelection(): Unit;
        /**
         *             Class member alias for {@link natives.CreateBuilding | CreateBuilding}
         *             
         */
        public createBuilding(unitTypeId: number, x: number, y: number, facing: number): Unit;
        /**
         *             Class member alias for {@link natives.CreateBuildingEx | CreateBuildingEx}
         *             
         */
        public createBuildingEx(unitTypeId: number, x: number, y: number, facing: number, isAutoBuild: boolean, workersCanAssist: boolean): Unit;
        /**
         *             Class member alias for {@link natives.CreateIllusion | CreateIllusion}
         *             
         */
        public createIllusion(unitTypeId: number, x: number, y: number, facing: number): Unit;
        /**
         *             Class member alias for {@link natives.CreateIllusionAtLoc | CreateIllusionAtLoc}
         *             
         */
        public createIllusionAtLoc(unitTypeId: number, whichLocation: HLocation, facing: number): Unit;
        /**
         *             Class member alias for {@link natives.QueueNeutralImmediateOrderById | QueueNeutralImmediateOrderById}
         *             
         */
        public queueNeutralImmediateOrderById(neutralStructure: Unit, unitId: number): boolean;
        /**
         *             Class member alias for {@link natives.QueueNeutralPointOrderById | QueueNeutralPointOrderById}
         *             
         */
        public queueNeutralPointOrderById(neutralStructure: Unit, unitId: number, x: number, y: number): boolean;
        /**
         *             Class member alias for {@link natives.QueueNeutralTargetOrderById | QueueNeutralTargetOrderById}
         *             
         */
        public queueNeutralTargetOrderById(neutralStructure: Unit, unitId: number, target: Widget): boolean;
        /**
         *             Class member alias for {@link natives.BlzDecPlayerTechResearched | BlzDecPlayerTechResearched}
         *             
         */
        public blzDecTechResearched(techid: number, levels: number): void;
        /**
         *             Class member alias for {@link natives.BlzDisplayChatMessage | BlzDisplayChatMessage}
         *             
         */
        public blzDisplayChatMessage(recipient: number, message: string): void;
        /**
         *             Class member alias for {@link natives.BlzCreateUnitWithSkin | BlzCreateUnitWithSkin}
         *             
         */
        public blzCreateUnitWithSkin(unitid: number, x: number, y: number, face: number, skinId: number): Unit;
        /**
         *             Class member alias for {@link natives.BlzGetPlayerTownHallCount | BlzGetPlayerTownHallCount}
         *             
         */
        public blzGetTownHallCount(): number;
        /**
         *             Class member alias for {@link natives.BlzQueueNeutralImmediateOrderById | BlzQueueNeutralImmediateOrderById}
         *             
         */
        public blzQueueNeutralImmediateOrderById(neutralStructure: Unit, unitId: number): boolean;
        /**
         *             Class member alias for {@link natives.BlzQueueNeutralPointOrderById | BlzQueueNeutralPointOrderById}
         *             
         */
        public blzQueueNeutralPointOrderById(neutralStructure: Unit, unitId: number, x: number, y: number): boolean;
        /**
         *             Class member alias for {@link natives.BlzQueueNeutralTargetOrderById | BlzQueueNeutralTargetOrderById}
         *             
         */
        public blzQueueNeutralTargetOrderById(neutralStructure: Unit, unitId: number, target: Widget): boolean;
    }

    interface DestructableEventMap extends WidgetEventMap {
    }

    class Destructable<T extends DestructableEventMap = DestructableEventMap> extends Widget<T> {
        constructor(destructableHandle: HandleHolder<"item">);
        constructor(destructableHandle: Destructable);
        constructor(destructableId: number, x: number, y: number, face: number, scale: number, variation: number);
        /**
         *             Class chain property alias for {@link natives.IsDestructableInvulnerable | IsDestructableInvulnerable} and {@link natives.SetDestructableInvulnerable | SetDestructableInvulnerable}
         *             
         */
        public invulnerable(): boolean;
        public invulnerable<T = boolean>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetDestructableX | GetDestructableX} and {@link natives.SetDestructableX | SetDestructableX}
         *             
         */
        public x(): number;
        public x<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetDestructableY | GetDestructableY} and {@link natives.SetDestructableY | SetDestructableY}
         *             
         */
        public y(): number;
        public y<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetDestructableLife | GetDestructableLife} and {@link natives.SetDestructableLife | SetDestructableLife}
         *             
         */
        public life(): number;
        public life<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetDestructableMaxLife | GetDestructableMaxLife} and {@link natives.SetDestructableMaxLife | SetDestructableMaxLife}
         *             
         */
        public maxLife(): number;
        public maxLife<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetDestructableOccluderHeight | GetDestructableOccluderHeight} and {@link natives.SetDestructableOccluderHeight | SetDestructableOccluderHeight}
         *             
         */
        public occluderHeight(): number;
        public occluderHeight<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetDestructableVariation | GetDestructableVariation} and {@link natives.SetDestructableVariation | SetDestructableVariation}
         *             
         */
        public variation(): number;
        public variation<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.IsDestructableBlighted | IsDestructableBlighted} and {@link natives.SetDestructableBlighted | SetDestructableBlighted}
         *             
         */
        public blighted(): boolean;
        public blighted<T = boolean>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetDestructablePositionLoc | GetDestructablePositionLoc} and {@link natives.SetDestructablePositionLoc | SetDestructablePositionLoc}
         *             
         */
        public positionLoc(): HLocation;
        public positionLoc<T = HLocation>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetDestructableTimeScale | GetDestructableTimeScale} and {@link natives.SetDestructableTimeScale | SetDestructableTimeScale}
         *             
         */
        public timeScale(): number;
        public timeScale<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetDestructableScale | GetDestructableScale} and {@link natives.SetDestructableTimeScale | SetDestructableTimeScale}
         *             
         */
        public scale(): number;
        public scale<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetDestructableYaw | GetDestructableYaw} and {@link natives.SetDestructableYaw | SetDestructableYaw}
         *             
         */
        public yaw(): number;
        public yaw<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetDestructablePitch | GetDestructablePitch} and {@link natives.SetDestructablePitch | SetDestructablePitch}
         *             
         */
        public pitch(): number;
        public pitch<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetDestructableRoll | GetDestructableRoll} and {@link natives.SetDestructableRoll | SetDestructableRoll}
         *             
         */
        public roll(): number;
        public roll<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetDestructableModel | GetDestructableModel} and {@link natives.SetDestructableModel | SetDestructableModel}
         *             
         */
        public model(): string;
        public model<T = string>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetDestructableAnimationOffsetPercent | GetDestructableAnimationOffsetPercent} and {@link natives.SetDestructableAnimationOffsetPercent | SetDestructableAnimationOffsetPercent}
         *             
         */
        public animationOffsetPercent(): number;
        public animationOffsetPercent<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class member alias for {@link natives.RemoveDestructable | RemoveDestructable}
         *             
         */
        public remove(): void;
        /**
         *             Class member alias for {@link natives.KillDestructable | KillDestructable}
         *             
         */
        public kill(): void;
        /**
         *             Class member alias for {@link natives.GetDestructableTypeId | GetDestructableTypeId}
         *             
         */
        public getTypeId(): number;
        /**
         *             Class member alias for {@link natives.DestructableRestoreLife | DestructableRestoreLife}
         *             
         */
        public restoreLife(life: number, birth: boolean): void;
        /**
         *             Class member alias for {@link natives.QueueDestructableAnimation | QueueDestructableAnimation}
         *             
         */
        public queueAnimation(whichAnimation: string): void;
        /**
         *             Class member alias for {@link natives.ShowDestructable | ShowDestructable}
         *             
         */
        public show(flag: boolean): void;
        /**
         *             Class member alias for {@link natives.GetDestructableName | GetDestructableName}
         *             
         */
        public getName(): string;
        /**
         *             Class member alias for {@link natives.SetDestructableVariationEx | SetDestructableVariationEx}
         *             
         */
        public setVariationEx(variation: number, ignoreStateCheck: boolean): void;
        /**
         *             Class member alias for {@link natives.GetDestructableSprite | GetDestructableSprite}
         *             
         */
        public getSprite(): HandleHolder<"sprite">;
        /**
         *             Class member alias for {@link natives.SetDestructablePositionWithZ | SetDestructablePositionWithZ}
         *             
         */
        public setPositionWithZ(x: number, y: number, z: number): void;
        /**
         *             Class member alias for {@link natives.SetDestructablePosition | SetDestructablePosition}
         *             
         */
        public setPosition(x: number, y: number): void;
        /**
         *             Class member alias for {@link natives.GetDestructableScreenX | GetDestructableScreenX}
         *             
         */
        public getScreenX(): number;
        /**
         *             Class member alias for {@link natives.GetDestructableScreenY | GetDestructableScreenY}
         *             
         */
        public getScreenY(): number;
        /**
         *             Class member alias for {@link natives.GetDestructableVertexColour | GetDestructableVertexColour}
         *             
         */
        public getVertexColour(): number;
        /**
         *             Class member alias for {@link natives.SetDestructableVertexColour | SetDestructableVertexColour}
         *             
         */
        public setVertexColour(red: number, green: number, blue: number, alpha: number): void;
        /**
         *             Class member alias for {@link natives.GetDestructableFacing | GetDestructableFacing}
         *             
         */
        public getFacing(): number;
        /**
         *             Class member alias for {@link natives.SetDestructableFacing | SetDestructableFacing}
         *             
         */
        public setFacing(facing: number, isInstant: boolean): void;
        /**
         *             Class member alias for {@link natives.SetDestructableMatrixScale | SetDestructableMatrixScale}
         *             
         */
        public setMatrixScale(x: number, y: number, z: number): void;
        /**
         *             Class member alias for {@link natives.ResetDestructableMatrix | ResetDestructableMatrix}
         *             
         */
        public resetMatrix(): void;
        /**
         *             Class member alias for {@link natives.SetDestructableOrientationEx | SetDestructableOrientationEx}
         *             
         */
        public setOrientationEx(yaw: number, pitch: number, roll: number, eulerOrder: number): void;
        /**
         *             Class member alias for {@link natives.SetDestructableOrientation | SetDestructableOrientation}
         *             
         */
        public setOrientation(yaw: number, pitch: number, roll: number): void;
        /**
         *             Class member alias for {@link natives.SetDestructableModelEx | SetDestructableModelEx}
         *             
         */
        public setModelEx(modelFile: string, playerId: number): void;
        /**
         *             Class member alias for {@link natives.SetDestructableMaterialTexture | SetDestructableMaterialTexture}
         *             
         */
        public setMaterialTexture(textureName: string, materialId: number, textureIndex: number): void;
        /**
         *             Class member alias for {@link natives.SetDestructableTexture | SetDestructableTexture}
         *             
         */
        public setTexture(textureName: string, textureIndex: number): void;
        /**
         *             Class member alias for {@link natives.SetDestructableReplaceableTexture | SetDestructableReplaceableTexture}
         *             
         */
        public setReplaceableTexture(textureName: string, textureIndex: number): void;
        /**
         *             Class member alias for {@link natives.GetDestructableModelObjectX | GetDestructableModelObjectX}
         *             
         */
        public getModelObjectX(whichObject: string): number;
        /**
         *             Class member alias for {@link natives.GetDestructableModelObjectY | GetDestructableModelObjectY}
         *             
         */
        public getModelObjectY(whichObject: string): number;
        /**
         *             Class member alias for {@link natives.GetDestructableModelObjectZ | GetDestructableModelObjectZ}
         *             
         */
        public getModelObjectZ(whichObject: string): number;
        /**
         *             Class member alias for {@link natives.GetDestructableModelObjectPositionLoc | GetDestructableModelObjectPositionLoc}
         *             
         */
        public getModelObjectPositionLoc(whichObject: string): HLocation;
        /**
         *             Class member alias for {@link natives.GetDestructableCurrentAnimationId | GetDestructableCurrentAnimationId}
         *             
         */
        public getCurrentAnimationId(): number;
        /**
         *             Class member alias for {@link natives.GetDestructableCurrentAnimationName | GetDestructableCurrentAnimationName}
         *             
         */
        public getCurrentAnimationName(): string;
        /**
         *             Class member alias for {@link natives.SetDestructableAnimationWithRarityByIndex | SetDestructableAnimationWithRarityByIndex}
         *             
         */
        public setAnimationWithRarityByIndex(animIndex: number, rarity: HandleHolder<"raritycontrol">): void;
        /**
         *             Class member alias for {@link natives.SetDestructableAnimationWithRarity | SetDestructableAnimationWithRarity}
         *             
         */
        public setAnimationWithRarity(animationName: string, rarity: HandleHolder<"raritycontrol">): void;
        /**
         *             Class member alias for {@link natives.QueueDestructableAnimationByIndex | QueueDestructableAnimationByIndex}
         *             
         */
        public queueAnimationByIndex(animIndex: number): void;
    }

    interface ItemEventMap extends WidgetEventMap {
    }

    class Item<T extends ItemEventMap = ItemEventMap> extends Widget<T> {
        constructor(itemHandle: HandleHolder<"item">);
        constructor(itemHandle: Item);
        constructor(itemId: number, x: number, y: number);
        /**
         *             Class chain property alias for {@link natives.IsItemInvulnerable | IsItemInvulnerable} and {@link natives.SetItemInvulnerable | SetItemInvulnerable}
         *             
         */
        public invulnerable(): boolean;
        public invulnerable<T = boolean>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.IsItemVisible | IsItemVisible} and {@link natives.SetItemVisible | SetItemVisible}
         *             
         */
        public visible(): boolean;
        public visible<T = boolean>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.IsItemPawnable | IsItemPawnable} and {@link natives.SetItemPawnable | SetItemPawnable}
         *             
         */
        public pawnable(): boolean;
        public pawnable<T = boolean>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetItemCharges | GetItemCharges} and {@link natives.SetItemCharges | SetItemCharges}
         *             
         */
        public charges(): number;
        public charges<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetItemUserData | GetItemUserData} and {@link natives.SetItemUserData | SetItemUserData}
         *             
         */
        public userData(): number;
        public userData<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetItemLife | GetItemLife} and {@link natives.SetItemLife | SetItemLife}
         *             
         */
        public life(): number;
        public life<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetItemMaxLife | GetItemMaxLife} and {@link natives.SetItemMaxLife | SetItemMaxLife}
         *             
         */
        public maxLife(): number;
        public maxLife<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetItemCooldown | GetItemCooldown} and {@link natives.SetItemCooldown | SetItemCooldown}
         *             
         */
        public cooldown(): number;
        public cooldown<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetItemRemainingCooldown | GetItemRemainingCooldown} and {@link natives.SetItemRemainingCooldown | SetItemRemainingCooldown}
         *             
         */
        public remainingCooldown(): number;
        public remainingCooldown<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetItemTimeScale | GetItemTimeScale} and {@link natives.SetItemTimeScale | SetItemTimeScale}
         *             
         */
        public timeScale(): number;
        public timeScale<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetItemScale | GetItemScale} and {@link natives.SetItemTimeScale | SetItemTimeScale}
         *             
         */
        public scale(): number;
        public scale<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetItemYaw | GetItemYaw} and {@link natives.SetItemYaw | SetItemYaw}
         *             
         */
        public yaw(): number;
        public yaw<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetItemPitch | GetItemPitch} and {@link natives.SetItemPitch | SetItemPitch}
         *             
         */
        public pitch(): number;
        public pitch<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetItemRoll | GetItemRoll} and {@link natives.SetItemRoll | SetItemRoll}
         *             
         */
        public roll(): number;
        public roll<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetItemModel | GetItemModel} and {@link natives.SetItemModel | SetItemModel}
         *             
         */
        public model(): string;
        public model<T = string>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetItemAnimationOffsetPercent | GetItemAnimationOffsetPercent} and {@link natives.SetItemAnimationOffsetPercent | SetItemAnimationOffsetPercent}
         *             
         */
        public animationOffsetPercent(): number;
        public animationOffsetPercent<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class member alias for {@link natives.RemoveItem | RemoveItem}
         *             
         */
        public remove(): void;
        /**
         *             Class member alias for {@link natives.GetItemPlayer | GetItemPlayer}
         *             
         */
        public getPlayer(): Player;
        /**
         *             Class member alias for {@link natives.GetItemTypeId | GetItemTypeId}
         *             
         */
        public getTypeId(): number;
        /**
         *             Class member alias for {@link natives.GetItemX | GetItemX}
         *             
         */
        public getX(): number;
        /**
         *             Class member alias for {@link natives.GetItemY | GetItemY}
         *             
         */
        public getY(): number;
        /**
         *             Class member alias for {@link natives.SetItemPosition | SetItemPosition}
         *             
         */
        public setPosition(x: number, y: number): void;
        /**
         *             Class member alias for {@link natives.SetItemPlayer | SetItemPlayer}
         *             
         */
        public setPlayer(whichPlayer: Player, changeColor: boolean): void;
        /**
         *             Class member alias for {@link natives.IsItemOwned | IsItemOwned}
         *             
         */
        public isOwned(): boolean;
        /**
         *             Class member alias for {@link natives.IsItemPowerup | IsItemPowerup}
         *             
         */
        public isPowerup(): boolean;
        /**
         *             Class member alias for {@link natives.IsItemSellable | IsItemSellable}
         *             
         */
        public isSellable(): boolean;
        /**
         *             Class member alias for {@link natives.GetItemLevel | GetItemLevel}
         *             
         */
        public getLevel(): number;
        /**
         *             Class member alias for {@link natives.GetItemType | GetItemType}
         *             
         */
        public getType(): HandleHolder<"itemtype">;
        /**
         *             Class member alias for {@link natives.GetItemName | GetItemName}
         *             
         */
        public getName(): string;
        /**
         *             Class member alias for {@link natives.GetItemIntegerField | GetItemIntegerField}
         *             
         */
        public getIntegerField(whichField: HandleHolder<"itemintegerfield">): number;
        /**
         *             Class member alias for {@link natives.SetItemIntegerField | SetItemIntegerField}
         *             
         */
        public setIntegerField(whichField: HandleHolder<"itemintegerfield">, value: number): boolean;
        /**
         *             Class member alias for {@link natives.GetItemBooleanField | GetItemBooleanField}
         *             
         */
        public getBooleanField(whichField: HandleHolder<"itembooleanfield">): boolean;
        /**
         *             Class member alias for {@link natives.SetItemBooleanField | SetItemBooleanField}
         *             
         */
        public setBooleanField(whichField: HandleHolder<"itembooleanfield">, value: boolean): boolean;
        /**
         *             Class member alias for {@link natives.GetItemRealField | GetItemRealField}
         *             
         */
        public getRealField(whichField: HandleHolder<"itemrealfield">): number;
        /**
         *             Class member alias for {@link natives.SetItemRealField | SetItemRealField}
         *             
         */
        public setRealField(whichField: HandleHolder<"itemrealfield">, value: number): boolean;
        /**
         *             Class member alias for {@link natives.GetItemStringField | GetItemStringField}
         *             
         */
        public getStringField(whichField: HandleHolder<"itemstringfield">): string;
        /**
         *             Class member alias for {@link natives.SetItemStringField | SetItemStringField}
         *             
         */
        public setStringField(whichField: HandleHolder<"itemstringfield">, value: string): boolean;
        /**
         *             Class member alias for {@link natives.GetItemSprite | GetItemSprite}
         *             
         */
        public getSprite(): HandleHolder<"sprite">;
        /**
         *             Class member alias for {@link natives.GetItemScreenX | GetItemScreenX}
         *             
         */
        public getScreenX(): number;
        /**
         *             Class member alias for {@link natives.GetItemScreenY | GetItemScreenY}
         *             
         */
        public getScreenY(): number;
        /**
         *             Class member alias for {@link natives.GetItemAbility | GetItemAbility}
         *             
         */
        public getAbility(abilityId: number): HandleHolder<"ability">;
        /**
         *             Class member alias for {@link natives.GetItemAbilityByIndex | GetItemAbilityByIndex}
         *             
         */
        public getAbilityByIndex(abilityIndex: number): HandleHolder<"ability">;
        /**
         *             Class member alias for {@link natives.GetItemVertexColour | GetItemVertexColour}
         *             
         */
        public getVertexColour(): number;
        /**
         *             Class member alias for {@link natives.SetItemVertexColour | SetItemVertexColour}
         *             
         */
        public setVertexColour(red: number, green: number, blue: number, alpha: number): void;
        /**
         *             Class member alias for {@link natives.GetItemFacing | GetItemFacing}
         *             
         */
        public getFacing(): number;
        /**
         *             Class member alias for {@link natives.SetItemFacing | SetItemFacing}
         *             
         */
        public setFacing(facing: number, isInstant: boolean): void;
        /**
         *             Class member alias for {@link natives.SetItemMatrixScale | SetItemMatrixScale}
         *             
         */
        public setMatrixScale(x: number, y: number, z: number): void;
        /**
         *             Class member alias for {@link natives.ResetItemMatrix | ResetItemMatrix}
         *             
         */
        public resetMatrix(): void;
        /**
         *             Class member alias for {@link natives.SetItemOrientationEx | SetItemOrientationEx}
         *             
         */
        public setOrientationEx(yaw: number, pitch: number, roll: number, eulerOrder: number): void;
        /**
         *             Class member alias for {@link natives.SetItemOrientation | SetItemOrientation}
         *             
         */
        public setOrientation(yaw: number, pitch: number, roll: number): void;
        /**
         *             Class member alias for {@link natives.SetItemModelEx | SetItemModelEx}
         *             
         */
        public setModelEx(modelFile: string, playerColourId: number): void;
        /**
         *             Class member alias for {@link natives.SetItemMaterialTexture | SetItemMaterialTexture}
         *             
         */
        public setMaterialTexture(textureName: string, materialId: number, textureIndex: number): void;
        /**
         *             Class member alias for {@link natives.SetItemTexture | SetItemTexture}
         *             
         */
        public setTexture(textureName: string, textureIndex: number): void;
        /**
         *             Class member alias for {@link natives.SetItemReplaceableTexture | SetItemReplaceableTexture}
         *             
         */
        public setReplaceableTexture(textureName: string, textureIndex: number): void;
        /**
         *             Class member alias for {@link natives.GetItemModelObjectX | GetItemModelObjectX}
         *             
         */
        public getModelObjectX(whichObject: string): number;
        /**
         *             Class member alias for {@link natives.GetItemModelObjectY | GetItemModelObjectY}
         *             
         */
        public getModelObjectY(whichObject: string): number;
        /**
         *             Class member alias for {@link natives.GetItemModelObjectZ | GetItemModelObjectZ}
         *             
         */
        public getModelObjectZ(whichObject: string): number;
        /**
         *             Class member alias for {@link natives.GetItemModelObjectPositionLoc | GetItemModelObjectPositionLoc}
         *             
         */
        public getModelObjectPositionLoc(whichObject: string): HLocation;
        /**
         *             Class member alias for {@link natives.GetItemCurrentAnimationId | GetItemCurrentAnimationId}
         *             
         */
        public getCurrentAnimationId(): number;
        /**
         *             Class member alias for {@link natives.GetItemCurrentAnimationName | GetItemCurrentAnimationName}
         *             
         */
        public getCurrentAnimationName(): string;
        /**
         *             Class member alias for {@link natives.SetItemAnimationWithRarityByIndex | SetItemAnimationWithRarityByIndex}
         *             
         */
        public setAnimationWithRarityByIndex(animIndex: number, rarity: HandleHolder<"raritycontrol">): void;
        /**
         *             Class member alias for {@link natives.SetItemAnimationWithRarity | SetItemAnimationWithRarity}
         *             
         */
        public setAnimationWithRarity(animationName: string, rarity: HandleHolder<"raritycontrol">): void;
        /**
         *             Class member alias for {@link natives.QueueItemAnimationByIndex | QueueItemAnimationByIndex}
         *             
         */
        public queueAnimationByIndex(animIndex: number): void;
        /**
         *             Class member alias for {@link natives.QueueItemAnimation | QueueItemAnimation}
         *             
         */
        public queueAnimation(animationName: string): void;
        /**
         *             Class member alias for {@link natives.BlzSetItemName | BlzSetItemName}
         *             
         */
        public blzSetName(name: string): void;
        /**
         *             Class member alias for {@link natives.BlzSetItemDescription | BlzSetItemDescription}
         *             
         */
        public blzSetDescription(description: string): void;
        /**
         *             Class member alias for {@link natives.BlzGetItemDescription | BlzGetItemDescription}
         *             
         */
        public blzGetDescription(): string;
        /**
         *             Class member alias for {@link natives.BlzSetItemTooltip | BlzSetItemTooltip}
         *             
         */
        public blzSetTooltip(tooltip: string): void;
        /**
         *             Class member alias for {@link natives.BlzGetItemTooltip | BlzGetItemTooltip}
         *             
         */
        public blzGetTooltip(): string;
        /**
         *             Class member alias for {@link natives.BlzSetItemExtendedTooltip | BlzSetItemExtendedTooltip}
         *             
         */
        public blzSetExtendedTooltip(extendedTooltip: string): void;
        /**
         *             Class member alias for {@link natives.BlzGetItemExtendedTooltip | BlzGetItemExtendedTooltip}
         *             
         */
        public blzGetExtendedTooltip(): string;
        /**
         *             Class member alias for {@link natives.BlzSetItemIconPath | BlzSetItemIconPath}
         *             
         */
        public blzSetIconPath(iconPath: string): void;
        /**
         *             Class member alias for {@link natives.BlzGetItemIconPath | BlzGetItemIconPath}
         *             
         */
        public blzGetIconPath(): string;
        /**
         *             Class member alias for {@link natives.BlzGetItemAbilityByIndex | BlzGetItemAbilityByIndex}
         *             
         */
        public blzGetAbilityByIndex(index: number): HandleHolder<"ability">;
        /**
         *             Class member alias for {@link natives.BlzGetItemAbility | BlzGetItemAbility}
         *             
         */
        public blzGetAbility(abilCode: number): HandleHolder<"ability">;
        /**
         *             Class member alias for {@link natives.BlzItemAddAbility | BlzItemAddAbility}
         *             
         */
        public blzAddAbility(abilCode: number): boolean;
        /**
         *             Class member alias for {@link natives.BlzGetItemBooleanField | BlzGetItemBooleanField}
         *             
         */
        public blzGetBooleanField(whichField: HandleHolder<"itembooleanfield">): boolean;
        /**
         *             Class member alias for {@link natives.BlzGetItemIntegerField | BlzGetItemIntegerField}
         *             
         */
        public blzGetIntegerField(whichField: HandleHolder<"itemintegerfield">): number;
        /**
         *             Class member alias for {@link natives.BlzGetItemRealField | BlzGetItemRealField}
         *             
         */
        public blzGetRealField(whichField: HandleHolder<"itemrealfield">): number;
        /**
         *             Class member alias for {@link natives.BlzGetItemStringField | BlzGetItemStringField}
         *             
         */
        public blzGetStringField(whichField: HandleHolder<"itemstringfield">): string;
        /**
         *             Class member alias for {@link natives.BlzSetItemBooleanField | BlzSetItemBooleanField}
         *             
         */
        public blzSetBooleanField(whichField: HandleHolder<"itembooleanfield">, value: boolean): boolean;
        /**
         *             Class member alias for {@link natives.BlzSetItemIntegerField | BlzSetItemIntegerField}
         *             
         */
        public blzSetIntegerField(whichField: HandleHolder<"itemintegerfield">, value: number): boolean;
        /**
         *             Class member alias for {@link natives.BlzSetItemRealField | BlzSetItemRealField}
         *             
         */
        public blzSetRealField(whichField: HandleHolder<"itemrealfield">, value: number): boolean;
        /**
         *             Class member alias for {@link natives.BlzSetItemStringField | BlzSetItemStringField}
         *             
         */
        public blzSetStringField(whichField: HandleHolder<"itemstringfield">, value: string): boolean;
        /**
         *             Class member alias for {@link natives.BlzItemRemoveAbility | BlzItemRemoveAbility}
         *             
         */
        public blzRemoveAbility(abilCode: number): boolean;
        /**
         *             Class member alias for {@link natives.BlzGetItemSkin | BlzGetItemSkin}
         *             
         */
        public blzGetSkin(): number;
        /**
         *             Class member alias for {@link natives.BlzSetItemSkin | BlzSetItemSkin}
         *             
         */
        public blzSetSkin(skinId: number): void;
    }

    export type HUnitEvent = HandleHolder<"unitevent">;
    export type HTrigger = HandleHolder<"trigger">;
    export type HTriggerAction = HandleHolder<"triggeraction">;
    export type HEvent = HandleHolder<"event">;
    export type HUnitState = HandleHolder<"unitstate">;
    export type HLocation = HandleHolder<"location">;
    export type JassCodeCallback = () => number | void;

    const enum UnitState {
        Life = 0,
        MaxLife = 1,
        Mana = 2,
        MaxMana = 3
    }

    const enum LimitOp {
        lt = 0,
        le = 1,
        eq = 2,
        gt = 3,
        ge = 4
    }

    export type StateEventSymbol = typeof stateEvent;

    interface UnitEventMap extends WidgetEventMap {
        selected: (event: TriggerUnitEvent<"selected">) => void;
        deselected: (event: TriggerUnitEvent<"deselected">) => void;
        damaged: () => void;
        damaging: () => void;
        death: (event: UnitEventDeath) => void;
        decay: (event: TriggerUnitEvent<"decay">) => void;
        detected: (event: UnitEventDetected) => void;
        hidden: (event: TriggerUnitEvent<"hidden">) => void;
        stateLimit: (event: UnitEventStateLimit) => void;
        acquiredTarget: (event: UnitEventAcquiredTarget) => void;
        targetInRange: (event: UnitEventTargetInRange) => void;
        attacked: (event: UnitEventAttacked) => void;
        rescued: (event: UnitEventRescued) => void;
        constructCancel: (event: TriggerUnitEvent<"constructCancel">) => void;
        constructFinish: (event: UnitEventConstructFinish) => void;
        upgradeStart: (event: TriggerUnitEvent<"upgradeStart">) => void;
        upgradeCancel: (event: TriggerUnitEvent<"upgradeCancel">) => void;
        upgradeFinish: (event: TriggerUnitEvent<"upgradeFinish">) => void;
        trainStart: (event: UnitEventTrainStart) => void;
        trainCancel: (event: UnitEventTrainCancel) => void;
        trainFinish: (event: UnitEventTrainFinish) => void;
        researchStart: (event: UnitEventResearchStart) => void;
        researchCancel: (event: UnitEventResearchCancel) => void;
        researchFinish: (event: UnitEventResearchFinish) => void;
        issuedOrder: (event: UnitEventIssuedOrder) => void;
        issuedPointOrder: (event: UnitEventIssuedPointOrder) => void;
        issuedTargetOrder: (event: UnitEventIssuedTargetOrder) => void;
        heroLevel: (event: TriggerUnitEvent<"heroLevel">) => void;
        heroSkill: (event: UnitEventHeroSkill) => void;
        heroRevivable: (event: UnitEventHeroRevivable) => void;
        heroReviveStart: (event: UnitEventHeroReviveStart) => void;
        heroReviveCancel: (event: UnitEventHeroReviveCancel) => void;
        heroReviveFinish: (event: UnitEventHeroReviveFinish) => void;
        summon: (event: UnitEventSummon) => void;
        dropItem: (event: UnitEventDropItem) => void;
        pickupItem: (event: UnitEventPickupItem) => void;
        useItem: (event: UnitEventUseItem) => void;
        loaded: (event: UnitEventLoaded) => void;
        attackFinished: (event: TriggerUnitEvent<"attackFinished">) => void;
        decayFinished: (event: TriggerUnitEvent<"decayFinished">) => void;
        [stateEvent]: (event: UnitEventStateLimit, emiter: UnitStateEmiter) => void;
    }

    interface Unit {
    }

    class Unit<T extends UnitEventMap = UnitEventMap> extends Widget<T> {
        constructor(unitHandle: HandleHolder<"unit">);
        constructor(unitobject: Unit);
        constructor(unitobject: Player, unitId: number, x: number, y: number, facing: number);
        onUnitState(unitState: UnitState, limitOp: LimitOp, value: number, callback: UnitEventMap[StateEventSymbol]): UnitStateEmiter;
        static createIllusionFromUnit(unit: Unit, options?: UnitIllusionOptions): Unit<UnitEventMap>;
        onEmitterAddListener(event: UnitEventType | number | symbol, listener: (...args: any[]) => void): void;
        /**
         *             Class chain property alias for {@link natives.GetUnitAcquireRange | GetUnitAcquireRange} and {@link natives.SetUnitAcquireRange | SetUnitAcquireRange}
         *             
         */
        public acquireRange(): number;
        public acquireRange<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitTurnSpeed | GetUnitTurnSpeed} and {@link natives.SetUnitTurnSpeed | SetUnitTurnSpeed}
         *             
         */
        public turnSpeed(): number;
        public turnSpeed<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitPropWindow | GetUnitPropWindow} and {@link natives.SetUnitPropWindow | SetUnitPropWindow}
         *             
         */
        public propWindow(): number;
        public propWindow<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitX | GetUnitX} and {@link natives.SetUnitX | SetUnitX}
         *             
         */
        public x(): number;
        public x<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitY | GetUnitY} and {@link natives.SetUnitY | SetUnitY}
         *             
         */
        public y(): number;
        public y<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitLoc | GetUnitLoc} and {@link natives.SetUnitPositionLoc | SetUnitPositionLoc}
         *             
         */
        public loc(): HLocation;
        public loc<T = HLocation>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitFacing | GetUnitFacing} and {@link natives.SetUnitFacing | SetUnitFacing}
         *             
         */
        public facing(): number;
        public facing<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitMoveSpeed | GetUnitMoveSpeed} and {@link natives.SetUnitMoveSpeed | SetUnitMoveSpeed}
         *             
         */
        public moveSpeed(): number;
        public moveSpeed<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitTypeId | GetUnitTypeId} and {@link natives.SetUnitTypeId | SetUnitTypeId}
         *             
         */
        public typeId(): number;
        public typeId<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitUserData | GetUnitUserData} and {@link natives.SetUnitUserData | SetUnitUserData}
         *             
         */
        public userData(): number;
        public userData<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.IsUnitStunned | IsUnitStunned} and {@link natives.SetUnitStunned | SetUnitStunned}
         *             
         */
        public stunned(): boolean;
        public stunned<T = boolean>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.IsUnitInvulnerable | IsUnitInvulnerable} and {@link natives.SetUnitInvulnerable | SetUnitInvulnerable}
         *             
         */
        public invulnerable(): boolean;
        public invulnerable<T = boolean>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitRemainingTimedLife | GetUnitRemainingTimedLife} and {@link natives.SetUnitRemainingTimedLife | SetUnitRemainingTimedLife}
         *             
         */
        public remainingTimedLife(): number;
        public remainingTimedLife<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.IsUnitSelectable | IsUnitSelectable} and {@link natives.SetUnitSelectable | SetUnitSelectable}
         *             
         */
        public selectable(): boolean;
        public selectable<T = boolean>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.IsUnitTargetable | IsUnitTargetable} and {@link natives.SetUnitTargetable | SetUnitTargetable}
         *             
         */
        public targetable(): boolean;
        public targetable<T = boolean>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitCurrentSight | GetUnitCurrentSight} and {@link natives.SetUnitCurrentSight | SetUnitCurrentSight}
         *             
         */
        public currentSight(): number;
        public currentSight<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitAttackRemainingCooldown | GetUnitAttackRemainingCooldown} and {@link natives.SetUnitAttackRemainingCooldown | SetUnitAttackRemainingCooldown}
         *             
         */
        public attackRemainingCooldown(): number;
        public attackRemainingCooldown<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitAttackRemainingDamagePoint | GetUnitAttackRemainingDamagePoint} and {@link natives.SetUnitAttackRemainingDamagePoint | SetUnitAttackRemainingDamagePoint}
         *             
         */
        public attackRemainingDamagePoint(): number;
        public attackRemainingDamagePoint<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitAttackRemainingBackswing | GetUnitAttackRemainingBackswing} and {@link natives.SetUnitAttackRemainingBackswing | SetUnitAttackRemainingBackswing}
         *             
         */
        public attackRemainingBackswing(): number;
        public attackRemainingBackswing<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitAttackSpeed | GetUnitAttackSpeed} and {@link natives.SetUnitAttackSpeed | SetUnitAttackSpeed}
         *             
         */
        public attackSpeed(): number;
        public attackSpeed<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitArmourType | GetUnitArmourType} and {@link natives.SetUnitArmourType | SetUnitArmourType}
         *             
         */
        public armourType(): HandleHolder<"defensetype">;
        public armourType<T = HandleHolder<"defensetype">>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitArmour | GetUnitArmour} and {@link natives.SetUnitArmour | SetUnitArmour}
         *             
         */
        public armour(): number;
        public armour<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitTimeScale | GetUnitTimeScale} and {@link natives.SetUnitTimeScale | SetUnitTimeScale}
         *             
         */
        public timeScale(): number;
        public timeScale<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitBaseMoveSpeed | GetUnitBaseMoveSpeed} and {@link natives.SetUnitBaseMoveSpeed | SetUnitBaseMoveSpeed}
         *             
         */
        public baseMoveSpeed(): number;
        public baseMoveSpeed<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitBonusMoveSpeedPercent | GetUnitBonusMoveSpeedPercent} and {@link natives.SetUnitBonusMoveSpeedPercent | SetUnitBonusMoveSpeedPercent}
         *             
         */
        public bonusMoveSpeedPercent(): number;
        public bonusMoveSpeedPercent<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitCurrentLife | GetUnitCurrentLife} and {@link natives.SetUnitCurrentLife | SetUnitCurrentLife}
         *             
         */
        public currentLife(): number;
        public currentLife<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitMaxLife | GetUnitMaxLife} and {@link natives.SetUnitMaxLife | SetUnitMaxLife}
         *             
         */
        public maxLife(): number;
        public maxLife<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitLifeRegen | GetUnitLifeRegen} and {@link natives.SetUnitLifeRegen | SetUnitLifeRegen}
         *             
         */
        public lifeRegen(): number;
        public lifeRegen<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitCurrentMana | GetUnitCurrentMana} and {@link natives.SetUnitCurrentMana | SetUnitCurrentMana}
         *             
         */
        public currentMana(): number;
        public currentMana<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitMaxMana | GetUnitMaxMana} and {@link natives.SetUnitMaxMana | SetUnitMaxMana}
         *             
         */
        public maxMana(): number;
        public maxMana<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitManaRegen | GetUnitManaRegen} and {@link natives.SetUnitManaRegen | SetUnitManaRegen}
         *             
         */
        public manaRegen(): number;
        public manaRegen<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitPrimaryStat | GetUnitPrimaryStat} and {@link natives.SetUnitPrimaryStat | SetUnitPrimaryStat}
         *             
         */
        public primaryStat(): HandleHolder<"heroattribute">;
        public primaryStat<T = HandleHolder<"heroattribute">>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitModel | GetUnitModel} and {@link natives.SetUnitModel | SetUnitModel}
         *             
         */
        public model(): string;
        public model<T = string>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitCollisionType | GetUnitCollisionType} and {@link natives.SetUnitCollisionType | SetUnitCollisionType}
         *             
         */
        public collisionType(): HandleHolder<"collisiontype">;
        public collisionType<T = HandleHolder<"collisiontype">>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitPathingAIType | GetUnitPathingAIType} and {@link natives.SetUnitPathingAIType | SetUnitPathingAIType}
         *             
         */
        public pathingAIType(): HandleHolder<"pathingaitype">;
        public pathingAIType<T = HandleHolder<"pathingaitype">>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitPathingType | GetUnitPathingType} and {@link natives.SetUnitPathingType | SetUnitPathingType}
         *             
         */
        public pathingType(): HandleHolder<"pathingtype">;
        public pathingType<T = HandleHolder<"pathingtype">>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitItemSlots | GetUnitItemSlots} and {@link natives.SetUnitItemSlots | SetUnitItemSlots}
         *             
         */
        public itemSlots(): number;
        public itemSlots<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitStunCounter | GetUnitStunCounter} and {@link natives.SetUnitStunCounter | SetUnitStunCounter}
         *             
         */
        public stunCounter(): number;
        public stunCounter<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitAnimationOffsetPercent | GetUnitAnimationOffsetPercent} and {@link natives.SetUnitAnimationOffsetPercent | SetUnitAnimationOffsetPercent}
         *             
         */
        public animationOffsetPercent(): number;
        public animationOffsetPercent<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitYaw | GetUnitYaw} and {@link natives.SetUnitYaw | SetUnitYaw}
         *             
         */
        public yaw(): number;
        public yaw<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitPitch | GetUnitPitch} and {@link natives.SetUnitPitch | SetUnitPitch}
         *             
         */
        public pitch(): number;
        public pitch<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class chain property alias for {@link natives.GetUnitRoll | GetUnitRoll} and {@link natives.SetUnitRoll | SetUnitRoll}
         *             
         */
        public roll(): number;
        public roll<T = number>(value: T | ((this: this, currentValue: T) => T)): this;
        /**
         *             Class member alias for {@link natives.KillUnit | KillUnit}
         *             
         */
        public kill(): void;
        /**
         *             Class member alias for {@link natives.RemoveUnit | RemoveUnit}
         *             
         */
        public remove(): void;
        /**
         *             Class member alias for {@link natives.ShowUnit | ShowUnit}
         *             
         */
        public show(show: boolean): void;
        /**
         *             Class member alias for {@link natives.SetUnitState | SetUnitState}
         *             
         */
        public setState(whichUnitState: HUnitState, newVal: number): void;
        /**
         *             Class member alias for {@link natives.SetUnitPosition | SetUnitPosition}
         *             
         */
        public setPosition(newX: number, newY: number): void;
        /**
         *             Class member alias for {@link natives.SetUnitFacingTimed | SetUnitFacingTimed}
         *             
         */
        public setFacingTimed(facingAngle: number, duration: number): void;
        /**
         *             Class member alias for {@link natives.SetUnitFlyHeight | SetUnitFlyHeight}
         *             
         */
        public setFlyHeight(newHeight: number, rate: number): void;
        /**
         *             Class member alias for {@link natives.GetUnitFlyHeight | GetUnitFlyHeight}
         *             
         */
        public getFlyHeight(): number;
        /**
         *             Class member alias for {@link natives.GetUnitDefaultAcquireRange | GetUnitDefaultAcquireRange}
         *             
         */
        public getDefaultAcquireRange(): number;
        /**
         *             Class member alias for {@link natives.GetUnitDefaultTurnSpeed | GetUnitDefaultTurnSpeed}
         *             
         */
        public getDefaultTurnSpeed(): number;
        /**
         *             Class member alias for {@link natives.GetUnitDefaultPropWindow | GetUnitDefaultPropWindow}
         *             
         */
        public getDefaultPropWindow(): number;
        /**
         *             Class member alias for {@link natives.GetUnitDefaultFlyHeight | GetUnitDefaultFlyHeight}
         *             
         */
        public getDefaultFlyHeight(): number;
        /**
         *             Class member alias for {@link natives.SetUnitOwner | SetUnitOwner}
         *             
         */
        public setOwner(whichPlayer: Player, changeColor: boolean): void;
        /**
         *             Class member alias for {@link natives.SetUnitScale | SetUnitScale}
         *             
         */
        public setScale(scaleX: number, scaleY: number, scaleZ: number): void;
        /**
         *             Class member alias for {@link natives.SetUnitVertexColor | SetUnitVertexColor}
         *             
         */
        public setVertexColor(red: number, green: number, blue: number, alpha: number): void;
        /**
         *             Class member alias for {@link natives.QueueUnitAnimation | QueueUnitAnimation}
         *             
         */
        public queueAnimation(whichAnimation: string): void;
        /**
         *             Class member alias for {@link natives.SetUnitAnimationWithRarity | SetUnitAnimationWithRarity}
         *             
         */
        public setAnimationWithRarity(whichAnimation: string, rarity: HandleHolder<"raritycontrol">): void;
        /**
         *             Class member alias for {@link natives.AddUnitAnimationProperties | AddUnitAnimationProperties}
         *             
         */
        public addAnimationProperties(animProperties: string, add: boolean): void;
        /**
         *             Class member alias for {@link natives.SetUnitLookAt | SetUnitLookAt}
         *             
         */
        public setLookAt(whichBone: string, lookAtTarget: Unit, offsetX: number, offsetY: number, offsetZ: number): void;
        /**
         *             Class member alias for {@link natives.ResetUnitLookAt | ResetUnitLookAt}
         *             
         */
        public resetLookAt(): void;
        /**
         *             Class member alias for {@link natives.SetUnitRescuable | SetUnitRescuable}
         *             
         */
        public setRescuable(byWhichPlayer: Player, flag: boolean): void;
        /**
         *             Class member alias for {@link natives.SetHeroStr | SetHeroStr}
         *             
         */
        public setHeroStr(newStr: number, permanent: boolean): void;
        /**
         *             Class member alias for {@link natives.SetHeroAgi | SetHeroAgi}
         *             
         */
        public setHeroAgi(newAgi: number, permanent: boolean): void;
        /**
         *             Class member alias for {@link natives.SetHeroInt | SetHeroInt}
         *             
         */
        public setHeroInt(newInt: number, permanent: boolean): void;
        /**
         *             Class member alias for {@link natives.GetHeroStr | GetHeroStr}
         *             
         */
        public getHeroStr(includeBonuses: boolean): number;
        /**
         *             Class member alias for {@link natives.GetHeroAgi | GetHeroAgi}
         *             
         */
        public getHeroAgi(includeBonuses: boolean): number;
        /**
         *             Class member alias for {@link natives.GetHeroInt | GetHeroInt}
         *             
         */
        public getHeroInt(includeBonuses: boolean): number;
        /**
         *             Class member alias for {@link natives.UnitStripHeroLevel | UnitStripHeroLevel}
         *             
         */
        public stripHeroLevel(howManyLevels: number): boolean;
        /**
         *             Class member alias for {@link natives.GetHeroXP | GetHeroXP}
         *             
         */
        public getHeroXP(): number;
        /**
         *             Class member alias for {@link natives.SetHeroXP | SetHeroXP}
         *             
         */
        public setHeroXP(newXpVal: number, showEyeCandy: boolean): void;
        /**
         *             Class member alias for {@link natives.GetHeroSkillPoints | GetHeroSkillPoints}
         *             
         */
        public getHeroSkillPoints(): number;
        /**
         *             Class member alias for {@link natives.UnitModifySkillPoints | UnitModifySkillPoints}
         *             
         */
        public modifySkillPoints(skillPointDelta: number): boolean;
        /**
         *             Class member alias for {@link natives.AddHeroXP | AddHeroXP}
         *             
         */
        public addHeroXP(xpToAdd: number, showEyeCandy: boolean): void;
        /**
         *             Class member alias for {@link natives.SetHeroLevel | SetHeroLevel}
         *             
         */
        public setHeroLevel(level: number, showEyeCandy: boolean): void;
        /**
         *             Class member alias for {@link natives.GetHeroLevel | GetHeroLevel}
         *             
         */
        public getHeroLevel(): number;
        /**
         *             Class member alias for {@link natives.GetUnitLevel | GetUnitLevel}
         *             
         */
        public getLevel(): number;
        /**
         *             Class member alias for {@link natives.GetHeroProperName | GetHeroProperName}
         *             
         */
        public getHeroProperName(): string;
        /**
         *             Class member alias for {@link natives.SuspendHeroXP | SuspendHeroXP}
         *             
         */
        public suspendHeroXP(flag: boolean): void;
        /**
         *             Class member alias for {@link natives.IsSuspendedXP | IsSuspendedXP}
         *             
         */
        public isSuspendedXP(): boolean;
        /**
         *             Class member alias for {@link natives.SelectHeroSkill | SelectHeroSkill}
         *             
         */
        public selectHeroSkill(abilcode: number): void;
        /**
         *             Class member alias for {@link natives.GetUnitAbilityLevel | GetUnitAbilityLevel}
         *             
         */
        public getAbilityLevel(abilcode: number): number;
        /**
         *             Class member alias for {@link natives.DecUnitAbilityLevel | DecUnitAbilityLevel}
         *             
         */
        public decAbilityLevel(abilcode: number): number;
        /**
         *             Class member alias for {@link natives.IncUnitAbilityLevel | IncUnitAbilityLevel}
         *             
         */
        public incAbilityLevel(abilcode: number): number;
        /**
         *             Class member alias for {@link natives.SetUnitAbilityLevel | SetUnitAbilityLevel}
         *             
         */
        public setAbilityLevel(abilcode: number, level: number): number;
        /**
         *             Class member alias for {@link natives.ReviveHero | ReviveHero}
         *             
         */
        public reviveHero(x: number, y: number, doEyecandy: boolean): boolean;
        /**
         *             Class member alias for {@link natives.ReviveHeroLoc | ReviveHeroLoc}
         *             
         */
        public reviveHeroLoc(loc: HLocation, doEyecandy: boolean): boolean;
        /**
         *             Class member alias for {@link natives.PauseUnit | PauseUnit}
         *             
         */
        public pause(flag: boolean): void;
        /**
         *             Class member alias for {@link natives.IsUnitPaused | IsUnitPaused}
         *             
         */
        public isPaused(): boolean;
        /**
         *             Class member alias for {@link natives.SelectUnit | SelectUnit}
         *             
         */
        public select(flag: boolean): void;
        /**
         *             Class member alias for {@link natives.GetUnitPointValue | GetUnitPointValue}
         *             
         */
        public getPointValue(): number;
        /**
         *             Class member alias for {@link natives.UnitAddItem | UnitAddItem}
         *             
         */
        public addItem(whichItem: Item): boolean;
        /**
         *             Class member alias for {@link natives.UnitAddItemById | UnitAddItemById}
         *             
         */
        public addItemById(itemId: number): Item;
        /**
         *             Class member alias for {@link natives.UnitAddItemToSlotById | UnitAddItemToSlotById}
         *             
         */
        public addItemToSlotById(itemId: number, itemSlot: number): boolean;
        /**
         *             Class member alias for {@link natives.UnitRemoveItem | UnitRemoveItem}
         *             
         */
        public removeItem(whichItem: Item): void;
        /**
         *             Class member alias for {@link natives.UnitRemoveItemFromSlot | UnitRemoveItemFromSlot}
         *             
         */
        public removeItemFromSlot(itemSlot: number): Item;
        /**
         *             Class member alias for {@link natives.UnitHasItem | UnitHasItem}
         *             
         */
        public hasItem(whichItem: Item): boolean;
        /**
         *             Class member alias for {@link natives.UnitItemInSlot | UnitItemInSlot}
         *             
         */
        public itemInSlot(itemSlot: number): Item;
        /**
         *             Class member alias for {@link natives.UnitInventorySize | UnitInventorySize}
         *             
         */
        public inventorySize(): number;
        /**
         *             Class member alias for {@link natives.UnitDropItemPoint | UnitDropItemPoint}
         *             
         */
        public dropItemPoint(whichItem: Item, x: number, y: number): boolean;
        /**
         *             Class member alias for {@link natives.UnitDropItemSlot | UnitDropItemSlot}
         *             
         */
        public dropItemSlot(whichItem: Item, slot: number): boolean;
        /**
         *             Class member alias for {@link natives.UnitDropItemTarget | UnitDropItemTarget}
         *             
         */
        public dropItemTarget(whichItem: Item, target: Widget): boolean;
        /**
         *             Class member alias for {@link natives.UnitUseItem | UnitUseItem}
         *             
         */
        public useItem(whichItem: Item): boolean;
        /**
         *             Class member alias for {@link natives.UnitUseItemPoint | UnitUseItemPoint}
         *             
         */
        public useItemPoint(whichItem: Item, x: number, y: number): boolean;
        /**
         *             Class member alias for {@link natives.UnitUseItemTarget | UnitUseItemTarget}
         *             
         */
        public useItemTarget(whichItem: Item, target: Widget): boolean;
        /**
         *             Class member alias for {@link natives.GetUnitDefaultMoveSpeed | GetUnitDefaultMoveSpeed}
         *             
         */
        public getDefaultMoveSpeed(): number;
        /**
         *             Class member alias for {@link natives.GetUnitState | GetUnitState}
         *             
         */
        public getState(whichUnitState: HUnitState): number;
        /**
         *             Class member alias for {@link natives.GetOwningPlayer | GetOwningPlayer}
         *             
         */
        public getOwningPlayer(): Player;
        /**
         *             Class member alias for {@link natives.GetUnitRace | GetUnitRace}
         *             
         */
        public getRace(): HandleHolder<"race">;
        /**
         *             Class member alias for {@link natives.GetUnitName | GetUnitName}
         *             
         */
        public getName(): string;
        /**
         *             Class member alias for {@link natives.GetUnitFoodUsed | GetUnitFoodUsed}
         *             
         */
        public getFoodUsed(): number;
        /**
         *             Class member alias for {@link natives.GetUnitFoodMade | GetUnitFoodMade}
         *             
         */
        public getFoodMade(): number;
        /**
         *             Class member alias for {@link natives.GetUnitRallyPoint | GetUnitRallyPoint}
         *             
         */
        public getRallyPoint(): HLocation;
        /**
         *             Class member alias for {@link natives.GetUnitRallyUnit | GetUnitRallyUnit}
         *             
         */
        public getRallyUnit(): Unit;
        /**
         *             Class member alias for {@link natives.GetUnitRallyDestructable | GetUnitRallyDestructable}
         *             
         */
        public getRallyDestructable(): Destructable;
        /**
         *             Class member alias for {@link natives.IsUnitInGroup | IsUnitInGroup}
         *             
         */
        public isInGroup(whichGroup: HandleHolder<"group">): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitInForce | IsUnitInForce}
         *             
         */
        public isInForce(whichForce: HandleHolder<"force">): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitOwnedByPlayer | IsUnitOwnedByPlayer}
         *             
         */
        public isOwnedByPlayer(whichPlayer: Player): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitAlly | IsUnitAlly}
         *             
         */
        public isAlly(whichPlayer: Player): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitEnemy | IsUnitEnemy}
         *             
         */
        public isEnemy(whichPlayer: Player): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitVisible | IsUnitVisible}
         *             
         */
        public isVisible(whichPlayer: Player): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitDetected | IsUnitDetected}
         *             
         */
        public isDetected(whichPlayer: Player): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitInvisible | IsUnitInvisible}
         *             
         */
        public isInvisible(whichPlayer: Player): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitFogged | IsUnitFogged}
         *             
         */
        public isFogged(whichPlayer: Player): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitMasked | IsUnitMasked}
         *             
         */
        public isMasked(whichPlayer: Player): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitSelected | IsUnitSelected}
         *             
         */
        public isSelected(whichPlayer: Player): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitRace | IsUnitRace}
         *             
         */
        public isRace(whichRace: HandleHolder<"race">): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitType | IsUnitType}
         *             
         */
        public isType(whichUnitType: HandleHolder<"unittype">): boolean;
        /**
         *             Class member alias for {@link natives.IsUnit | IsUnit}
         *             
         */
        public is(whichSpecifiedUnit: Unit): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitInRange | IsUnitInRange}
         *             
         */
        public isInRange(otherUnit: Unit, distance: number): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitInRangeXY | IsUnitInRangeXY}
         *             
         */
        public isInRangeXY(x: number, y: number, distance: number): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitInRangeLoc | IsUnitInRangeLoc}
         *             
         */
        public isInRangeLoc(whichLocation: HLocation, distance: number): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitHidden | IsUnitHidden}
         *             
         */
        public isHidden(): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitIllusion | IsUnitIllusion}
         *             
         */
        public isIllusion(): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitInTransport | IsUnitInTransport}
         *             
         */
        public isInTransport(whichTransport: Unit): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitLoaded | IsUnitLoaded}
         *             
         */
        public isLoaded(): boolean;
        /**
         *             Class member alias for {@link natives.UnitShareVision | UnitShareVision}
         *             
         */
        public shareVision(whichPlayer: Player, share: boolean): void;
        /**
         *             Class member alias for {@link natives.UnitSuspendDecay | UnitSuspendDecay}
         *             
         */
        public suspendDecay(suspend: boolean): void;
        /**
         *             Class member alias for {@link natives.UnitAddType | UnitAddType}
         *             
         */
        public addType(whichUnitType: HandleHolder<"unittype">): boolean;
        /**
         *             Class member alias for {@link natives.UnitRemoveType | UnitRemoveType}
         *             
         */
        public removeType(whichUnitType: HandleHolder<"unittype">): boolean;
        /**
         *             Class member alias for {@link natives.UnitAddAbility | UnitAddAbility}
         *             
         */
        public addAbility(abilityId: number): boolean;
        /**
         *             Class member alias for {@link natives.UnitRemoveAbility | UnitRemoveAbility}
         *             
         */
        public removeAbility(abilityId: number): boolean;
        /**
         *             Class member alias for {@link natives.UnitMakeAbilityPermanent | UnitMakeAbilityPermanent}
         *             
         */
        public makeAbilityPermanent(permanent: boolean, abilityId: number): boolean;
        /**
         *             Class member alias for {@link natives.UnitRemoveBuffs | UnitRemoveBuffs}
         *             
         */
        public removeBuffs(removePositive: boolean, removeNegative: boolean): void;
        /**
         *             Class member alias for {@link natives.UnitRemoveBuffsEx | UnitRemoveBuffsEx}
         *             
         */
        public removeBuffsEx(removePositive: boolean, removeNegative: boolean, magic: boolean, physical: boolean, timedLife: boolean, aura: boolean, autoDispel: boolean): void;
        /**
         *             Class member alias for {@link natives.UnitHasBuffsEx | UnitHasBuffsEx}
         *             
         */
        public hasBuffsEx(removePositive: boolean, removeNegative: boolean, magic: boolean, physical: boolean, timedLife: boolean, aura: boolean, autoDispel: boolean): boolean;
        /**
         *             Class member alias for {@link natives.UnitCountBuffsEx | UnitCountBuffsEx}
         *             
         */
        public countBuffsEx(removePositive: boolean, removeNegative: boolean, magic: boolean, physical: boolean, timedLife: boolean, aura: boolean, autoDispel: boolean): number;
        /**
         *             Class member alias for {@link natives.UnitAddSleep | UnitAddSleep}
         *             
         */
        public addSleep(add: boolean): void;
        /**
         *             Class member alias for {@link natives.UnitCanSleep | UnitCanSleep}
         *             
         */
        public canSleep(): boolean;
        /**
         *             Class member alias for {@link natives.UnitAddSleepPerm | UnitAddSleepPerm}
         *             
         */
        public addSleepPerm(add: boolean): void;
        /**
         *             Class member alias for {@link natives.UnitCanSleepPerm | UnitCanSleepPerm}
         *             
         */
        public canSleepPerm(): boolean;
        /**
         *             Class member alias for {@link natives.UnitIsSleeping | UnitIsSleeping}
         *             
         */
        public isSleeping(): boolean;
        /**
         *             Class member alias for {@link natives.UnitWakeUp | UnitWakeUp}
         *             
         */
        public wakeUp(): void;
        /**
         *             Class member alias for {@link natives.UnitApplyTimedLife | UnitApplyTimedLife}
         *             
         */
        public applyTimedLife(buffId: number, duration: number): void;
        /**
         *             Class member alias for {@link natives.UnitIgnoreAlarm | UnitIgnoreAlarm}
         *             
         */
        public ignoreAlarm(flag: boolean): boolean;
        /**
         *             Class member alias for {@link natives.UnitIgnoreAlarmToggled | UnitIgnoreAlarmToggled}
         *             
         */
        public ignoreAlarmToggled(): boolean;
        /**
         *             Class member alias for {@link natives.UnitResetCooldown | UnitResetCooldown}
         *             
         */
        public resetCooldown(): void;
        /**
         *             Class member alias for {@link natives.UnitSetConstructionProgress | UnitSetConstructionProgress}
         *             
         */
        public setConstructionProgress(constructionPercentage: number): void;
        /**
         *             Class member alias for {@link natives.UnitSetUpgradeProgress | UnitSetUpgradeProgress}
         *             
         */
        public setUpgradeProgress(upgradePercentage: number): void;
        /**
         *             Class member alias for {@link natives.UnitPauseTimedLife | UnitPauseTimedLife}
         *             
         */
        public pauseTimedLife(flag: boolean): void;
        /**
         *             Class member alias for {@link natives.UnitSetUsesAltIcon | UnitSetUsesAltIcon}
         *             
         */
        public setUsesAltIcon(flag: boolean): void;
        /**
         *             Class member alias for {@link natives.UnitDamagePoint | UnitDamagePoint}
         *             
         */
        public damagePoint(delay: number, radius: number, x: number, y: number, amount: number, attack: boolean, ranged: boolean, attackType: HandleHolder<"attacktype">, damageType: HandleHolder<"damagetype">, weaponType: HandleHolder<"weapontype">): boolean;
        /**
         *             Class member alias for {@link natives.UnitDamageTarget | UnitDamageTarget}
         *             
         */
        public damageTarget(target: Widget, amount: number, attack: boolean, ranged: boolean, attackType: HandleHolder<"attacktype">, damageType: HandleHolder<"damagetype">, weaponType: HandleHolder<"weapontype">): boolean;
        /**
         *             Class member alias for {@link natives.IssueImmediateOrder | IssueImmediateOrder}
         *             
         */
        public issueImmediateOrder(order: string): boolean;
        /**
         *             Class member alias for {@link natives.IssueImmediateOrderById | IssueImmediateOrderById}
         *             
         */
        public issueImmediateOrderById(order: number): boolean;
        /**
         *             Class member alias for {@link natives.IssuePointOrder | IssuePointOrder}
         *             
         */
        public issuePointOrder(order: string, x: number, y: number): boolean;
        /**
         *             Class member alias for {@link natives.IssuePointOrderLoc | IssuePointOrderLoc}
         *             
         */
        public issuePointOrderLoc(order: string, whichLocation: HLocation): boolean;
        /**
         *             Class member alias for {@link natives.IssuePointOrderById | IssuePointOrderById}
         *             
         */
        public issuePointOrderById(order: number, x: number, y: number): boolean;
        /**
         *             Class member alias for {@link natives.IssuePointOrderByIdLoc | IssuePointOrderByIdLoc}
         *             
         */
        public issuePointOrderByIdLoc(order: number, whichLocation: HLocation): boolean;
        /**
         *             Class member alias for {@link natives.IssueTargetOrder | IssueTargetOrder}
         *             
         */
        public issueTargetOrder(order: string, targetWidget: Widget): boolean;
        /**
         *             Class member alias for {@link natives.IssueTargetOrderById | IssueTargetOrderById}
         *             
         */
        public issueTargetOrderById(order: number, targetWidget: Widget): boolean;
        /**
         *             Class member alias for {@link natives.IssueInstantPointOrder | IssueInstantPointOrder}
         *             
         */
        public issueInstantPointOrder(order: string, x: number, y: number, instantTargetWidget: Widget): boolean;
        /**
         *             Class member alias for {@link natives.IssueInstantPointOrderById | IssueInstantPointOrderById}
         *             
         */
        public issueInstantPointOrderById(order: number, x: number, y: number, instantTargetWidget: Widget): boolean;
        /**
         *             Class member alias for {@link natives.IssueInstantTargetOrder | IssueInstantTargetOrder}
         *             
         */
        public issueInstantTargetOrder(order: string, targetWidget: Widget, instantTargetWidget: Widget): boolean;
        /**
         *             Class member alias for {@link natives.IssueInstantTargetOrderById | IssueInstantTargetOrderById}
         *             
         */
        public issueInstantTargetOrderById(order: number, targetWidget: Widget, instantTargetWidget: Widget): boolean;
        /**
         *             Class member alias for {@link natives.IssueBuildOrder | IssueBuildOrder}
         *             
         */
        public issueBuildOrder(unitToBuild: string, x: number, y: number): boolean;
        /**
         *             Class member alias for {@link natives.IssueBuildOrderById | IssueBuildOrderById}
         *             
         */
        public issueBuildOrderById(unitId: number, x: number, y: number): boolean;
        /**
         *             Class member alias for {@link natives.GetUnitCurrentOrder | GetUnitCurrentOrder}
         *             
         */
        public getCurrentOrder(): number;
        /**
         *             Class member alias for {@link natives.SetResourceAmount | SetResourceAmount}
         *             
         */
        public setResourceAmount(amount: number): void;
        /**
         *             Class member alias for {@link natives.AddResourceAmount | AddResourceAmount}
         *             
         */
        public addResourceAmount(amount: number): void;
        /**
         *             Class member alias for {@link natives.GetResourceAmount | GetResourceAmount}
         *             
         */
        public getResourceAmount(): number;
        /**
         *             Class member alias for {@link natives.WaygateGetDestinationX | WaygateGetDestinationX}
         *             
         */
        public waygateGetDestinationX(): number;
        /**
         *             Class member alias for {@link natives.WaygateGetDestinationY | WaygateGetDestinationY}
         *             
         */
        public waygateGetDestinationY(): number;
        /**
         *             Class member alias for {@link natives.WaygateSetDestination | WaygateSetDestination}
         *             
         */
        public waygateSetDestination(x: number, y: number): void;
        /**
         *             Class member alias for {@link natives.WaygateActivate | WaygateActivate}
         *             
         */
        public waygateActivate(activate: boolean): void;
        /**
         *             Class member alias for {@link natives.WaygateIsActive | WaygateIsActive}
         *             
         */
        public waygateIsActive(): boolean;
        /**
         *             Class member alias for {@link natives.AddItemToStock | AddItemToStock}
         *             
         */
        public addItemToStock(itemId: number, currentStock: number, stockMax: number): void;
        /**
         *             Class member alias for {@link natives.AddUnitToStock | AddUnitToStock}
         *             
         */
        public addToStock(unitId: number, currentStock: number, stockMax: number): void;
        /**
         *             Class member alias for {@link natives.RemoveItemFromStock | RemoveItemFromStock}
         *             
         */
        public removeItemFromStock(itemId: number): void;
        /**
         *             Class member alias for {@link natives.RemoveUnitFromStock | RemoveUnitFromStock}
         *             
         */
        public removeFromStock(unitId: number): void;
        /**
         *             Class member alias for {@link natives.SetItemTypeSlots | SetItemTypeSlots}
         *             
         */
        public setItemTypeSlots(slots: number): void;
        /**
         *             Class member alias for {@link natives.UnitAddIndicator | UnitAddIndicator}
         *             
         */
        public addIndicator(red: number, green: number, blue: number, alpha: number): void;
        /**
         *             Class member alias for {@link natives.SetCameraTargetController | SetCameraTargetController}
         *             
         */
        public setCameraTargetController(xoffset: number, yoffset: number, inheritOrientation: boolean): void;
        /**
         *             Class member alias for {@link natives.SetCameraOrientController | SetCameraOrientController}
         *             
         */
        public setCameraOrientController(xoffset: number, yoffset: number): void;
        /**
         *             Class member alias for {@link natives.RemoveGuardPosition | RemoveGuardPosition}
         *             
         */
        public removeGuardPosition(): void;
        /**
         *             Class member alias for {@link natives.RecycleGuardPosition | RecycleGuardPosition}
         *             
         */
        public recycleGuardPosition(): void;
        /**
         *             Class member alias for {@link natives.EnumUnitAbilities | EnumUnitAbilities}
         *             
         */
        public enumAbilities(whichBoolexpr: HandleHolder<"boolexpr">, whichCode: JassCodeCallback): boolean;
        /**
         *             Class member alias for {@link natives.EnumUnitBuffs | EnumUnitBuffs}
         *             
         */
        public enumBuffs(whichBoolexpr: HandleHolder<"boolexpr">, whichCode: JassCodeCallback): boolean;
        /**
         *             Class member alias for {@link natives.StartItemCooldown | StartItemCooldown}
         *             
         */
        public startItemCooldown(whichItem: Item, cooldown: number): void;
        /**
         *             Class member alias for {@link natives.GetUnitIntegerField | GetUnitIntegerField}
         *             
         */
        public getIntegerField(whichField: HandleHolder<"unitintegerfield">): number;
        /**
         *             Class member alias for {@link natives.SetUnitIntegerField | SetUnitIntegerField}
         *             
         */
        public setIntegerField(whichField: HandleHolder<"unitintegerfield">, value: number): boolean;
        /**
         *             Class member alias for {@link natives.GetUnitBooleanField | GetUnitBooleanField}
         *             
         */
        public getBooleanField(whichField: HandleHolder<"unitbooleanfield">): boolean;
        /**
         *             Class member alias for {@link natives.SetUnitBooleanField | SetUnitBooleanField}
         *             
         */
        public setBooleanField(whichField: HandleHolder<"unitbooleanfield">, value: boolean): boolean;
        /**
         *             Class member alias for {@link natives.GetUnitRealField | GetUnitRealField}
         *             
         */
        public getRealField(whichField: HandleHolder<"unitrealfield">): number;
        /**
         *             Class member alias for {@link natives.SetUnitRealField | SetUnitRealField}
         *             
         */
        public setRealField(whichField: HandleHolder<"unitrealfield">, value: number): boolean;
        /**
         *             Class member alias for {@link natives.GetUnitStringField | GetUnitStringField}
         *             
         */
        public getStringField(whichField: HandleHolder<"unitstringfield">): string;
        /**
         *             Class member alias for {@link natives.SetUnitStringField | SetUnitStringField}
         *             
         */
        public setStringField(whichField: HandleHolder<"unitstringfield">, value: string): boolean;
        /**
         *             Class member alias for {@link natives.GetUnitWeaponIntegerField | GetUnitWeaponIntegerField}
         *             
         */
        public getWeaponIntegerField(whichField: HandleHolder<"unitweaponintegerfield">, index: number): number;
        /**
         *             Class member alias for {@link natives.SetUnitWeaponIntegerField | SetUnitWeaponIntegerField}
         *             
         */
        public setWeaponIntegerField(whichField: HandleHolder<"unitweaponintegerfield">, index: number, value: number): boolean;
        /**
         *             Class member alias for {@link natives.GetUnitWeaponBooleanField | GetUnitWeaponBooleanField}
         *             
         */
        public getWeaponBooleanField(whichField: HandleHolder<"unitweaponbooleanfield">, index: number): boolean;
        /**
         *             Class member alias for {@link natives.SetUnitWeaponBooleanField | SetUnitWeaponBooleanField}
         *             
         */
        public setWeaponBooleanField(whichField: HandleHolder<"unitweaponbooleanfield">, index: number, value: boolean): boolean;
        /**
         *             Class member alias for {@link natives.GetUnitWeaponRealField | GetUnitWeaponRealField}
         *             
         */
        public getWeaponRealField(whichField: HandleHolder<"unitweaponrealfield">, index: number): number;
        /**
         *             Class member alias for {@link natives.SetUnitWeaponRealField | SetUnitWeaponRealField}
         *             
         */
        public setWeaponRealField(whichField: HandleHolder<"unitweaponrealfield">, index: number, value: number): boolean;
        /**
         *             Class member alias for {@link natives.GetUnitWeaponStringField | GetUnitWeaponStringField}
         *             
         */
        public getWeaponStringField(whichField: HandleHolder<"unitweaponstringfield">, index: number): string;
        /**
         *             Class member alias for {@link natives.SetUnitWeaponStringField | SetUnitWeaponStringField}
         *             
         */
        public setWeaponStringField(whichField: HandleHolder<"unitweaponstringfield">, index: number, value: string): boolean;
        /**
         *             Class member alias for {@link natives.GetUnitSprite | GetUnitSprite}
         *             
         */
        public getSprite(): HandleHolder<"sprite">;
        /**
         *             Class member alias for {@link natives.GetUnitScreenX | GetUnitScreenX}
         *             
         */
        public getScreenX(): number;
        /**
         *             Class member alias for {@link natives.GetUnitScreenY | GetUnitScreenY}
         *             
         */
        public getScreenY(): number;
        /**
         *             Class member alias for {@link natives.IsUnitAlive | IsUnitAlive}
         *             
         */
        public isAlive(): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitDead | IsUnitDead}
         *             
         */
        public isDead(): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitMoving | IsUnitMoving}
         *             
         */
        public isMoving(): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitHero | IsUnitHero}
         *             
         */
        public isHero(): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitTower | IsUnitTower}
         *             
         */
        public isTower(): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitShop | IsUnitShop}
         *             
         */
        public isShop(): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitInventoryEnabled | IsUnitInventoryEnabled}
         *             
         */
        public isInventoryEnabled(): boolean;
        /**
         *             Class member alias for {@link natives.UnitEnableInventory | UnitEnableInventory}
         *             
         */
        public enableInventory(enable: boolean, ignoreErrorMessages: boolean): void;
        /**
         *             Class member alias for {@link natives.IsUnitMovementEnabled | IsUnitMovementEnabled}
         *             
         */
        public isMovementEnabled(): boolean;
        /**
         *             Class member alias for {@link natives.UnitEnableMovement | UnitEnableMovement}
         *             
         */
        public enableMovement(enable: boolean, fullDisable: boolean): void;
        /**
         *             Class member alias for {@link natives.IsUnitAttackEnabled | IsUnitAttackEnabled}
         *             
         */
        public isAttackEnabled(): boolean;
        /**
         *             Class member alias for {@link natives.UnitEnableAttack | UnitEnableAttack}
         *             
         */
        public enableAttack(enable: boolean, extraFlag: boolean): void;
        /**
         *             Class member alias for {@link natives.IsUnitStateNormal | IsUnitStateNormal}
         *             
         */
        public isStateNormal(additionalCheck: boolean): boolean;
        /**
         *             Class member alias for {@link natives.RedrawUnit | RedrawUnit}
         *             
         */
        public redraw(): void;
        /**
         *             Class member alias for {@link natives.UpdateUnitInfoBar | UpdateUnitInfoBar}
         *             
         */
        public updateInfoBar(): void;
        /**
         *             Class member alias for {@link natives.UnitUnapplyUpgrades | UnitUnapplyUpgrades}
         *             
         */
        public unapplyUpgrades(): void;
        /**
         *             Class member alias for {@link natives.UnitApplyUpgrades | UnitApplyUpgrades}
         *             
         */
        public applyUpgrades(): void;
        /**
         *             Class member alias for {@link natives.GetUnitAbility | GetUnitAbility}
         *             
         */
        public getAbility(aid: number): HandleHolder<"ability">;
        /**
         *             Class member alias for {@link natives.GetUnitAbilityByIndex | GetUnitAbilityByIndex}
         *             
         */
        public getAbilityByIndex(index: number): HandleHolder<"ability">;
        /**
         *             Class member alias for {@link natives.UnitAddAbilityEx | UnitAddAbilityEx}
         *             
         */
        public addAbilityEx(abilityId: number, checkForDuplicates: boolean): boolean;
        /**
         *             Class member alias for {@link natives.UnitRemoveAbilityEx | UnitRemoveAbilityEx}
         *             
         */
        public removeAbilityEx(abilityId: number, removeDuplicates: boolean): boolean;
        /**
         *             Class member alias for {@link natives.IsUnitAbilityVisible | IsUnitAbilityVisible}
         *             
         */
        public isAbilityVisible(abilityId: number): boolean;
        /**
         *             Class member alias for {@link natives.ShowUnitAbility | ShowUnitAbility}
         *             
         */
        public showAbility(abilityId: number, show: boolean): void;
        /**
         *             Class member alias for {@link natives.ShowUnitAbilityEx | ShowUnitAbilityEx}
         *             
         */
        public showAbilityEx(abilityId: number, show: boolean, checkDuplicates: boolean): void;
        /**
         *             Class member alias for {@link natives.DisableUnitAbility | DisableUnitAbility}
         *             
         */
        public disableAbility(abilityId: number, hide: boolean, disable: boolean): void;
        /**
         *             Class member alias for {@link natives.DisableUnitAbilityEx | DisableUnitAbilityEx}
         *             
         */
        public disableAbilityEx(abilityId: number, hide: boolean, disable: boolean, checkDuplicates: boolean): void;
        /**
         *             Class member alias for {@link natives.EnableUnitAbility | EnableUnitAbility}
         *             
         */
        public enableAbility(abilityId: number, show: boolean, enable: boolean): void;
        /**
         *             Class member alias for {@link natives.EnableUnitAbilityEx | EnableUnitAbilityEx}
         *             
         */
        public enableAbilityEx(abilityId: number, show: boolean, enable: boolean, checkDuplicates: boolean): void;
        /**
         *             Class member alias for {@link natives.UnitAddBuff | UnitAddBuff}
         *             
         */
        public addBuff(whichBuff: HandleHolder<"buff">): boolean;
        /**
         *             Class member alias for {@link natives.UnitAddBuffEx | UnitAddBuffEx}
         *             
         */
        public addBuffEx(whichBuff: HandleHolder<"buff">, checkForDuplicates: boolean): boolean;
        /**
         *             Class member alias for {@link natives.UnitAddBuffById | UnitAddBuffById}
         *             
         */
        public addBuffById(buffId: number): boolean;
        /**
         *             Class member alias for {@link natives.UnitAddBuffByIdEx | UnitAddBuffByIdEx}
         *             
         */
        public addBuffByIdEx(buffId: number, checkForDuplicates: boolean): boolean;
        /**
         *             Class member alias for {@link natives.GetUnitBuff | GetUnitBuff}
         *             
         */
        public getBuff(buffId: number): HandleHolder<"buff">;
        /**
         *             Class member alias for {@link natives.GetUnitBuffByIndex | GetUnitBuffByIndex}
         *             
         */
        public getBuffByIndex(index: number): HandleHolder<"buff">;
        /**
         *             Class member alias for {@link natives.GetUnitBuffLevel | GetUnitBuffLevel}
         *             
         */
        public getBuffLevel(buffId: number): number;
        /**
         *             Class member alias for {@link natives.UnitCancelTimedLife | UnitCancelTimedLife}
         *             
         */
        public cancelTimedLife(): void;
        /**
         *             Class member alias for {@link natives.IsUnitTruesightImmune | IsUnitTruesightImmune}
         *             
         */
        public isTruesightImmune(): boolean;
        /**
         *             Class member alias for {@link natives.SetUnitVisibleByPlayer | SetUnitVisibleByPlayer}
         *             
         */
        public setVisibleByPlayer(whichPlayer: Player, flag: boolean): void;
        /**
         *             Class member alias for {@link natives.SetUnitDetectableByPlayer | SetUnitDetectableByPlayer}
         *             
         */
        public setDetectableByPlayer(whichPlayer: Player, flag: boolean): void;
        /**
         *             Class member alias for {@link natives.GetUnitZ | GetUnitZ}
         *             
         */
        public getZ(): number;
        /**
         *             Class member alias for {@link natives.GetUnitDamageReduction | GetUnitDamageReduction}
         *             
         */
        public getDamageReduction(): number;
        /**
         *             Class member alias for {@link natives.GetUnitMagicResistByType | GetUnitMagicResistByType}
         *             
         */
        public getMagicResistByType(resistType: number): number;
        /**
         *             Class member alias for {@link natives.GetUnitEluneMagicResist | GetUnitEluneMagicResist}
         *             
         */
        public getEluneMagicResist(): number;
        /**
         *             Class member alias for {@link natives.GetUnitRunicMagicResist | GetUnitRunicMagicResist}
         *             
         */
        public getRunicMagicResist(): number;
        /**
         *             Class member alias for {@link natives.GetUnitTotalMagicResist | GetUnitTotalMagicResist}
         *             
         */
        public getTotalMagicResist(): number;
        /**
         *             Class member alias for {@link natives.IsUnitGatherer | IsUnitGatherer}
         *             
         */
        public isGatherer(): boolean;
        /**
         *             Class member alias for {@link natives.GetUnitCurrentResources | GetUnitCurrentResources}
         *             
         */
        public getCurrentResources(): number;
        /**
         *             Class member alias for {@link natives.UnitAttackTarget | UnitAttackTarget}
         *             
         */
        public attackTarget(whichTarget: Widget, ignoreDistance: boolean, isInstant: boolean): void;
        /**
         *             Class member alias for {@link natives.UnitAttackTargetPointZ | UnitAttackTargetPointZ}
         *             
         */
        public attackTargetPointZ(x: number, y: number, z: number, ignoreDistance: boolean, isInstant: boolean): void;
        /**
         *             Class member alias for {@link natives.UnitAttackTargetPoint | UnitAttackTargetPoint}
         *             
         */
        public attackTargetPoint(x: number, y: number, ignoreDistance: boolean, isInstant: boolean): void;
        /**
         *             Class member alias for {@link natives.UnitAttackTargetPointLoc | UnitAttackTargetPointLoc}
         *             
         */
        public attackTargetPointLoc(whichLocation: HLocation, ignoreDistance: boolean, isInstant: boolean): void;
        /**
         *             Class member alias for {@link natives.UnitAttackCancel | UnitAttackCancel}
         *             
         */
        public attackCancel(): boolean;
        /**
         *             Class member alias for {@link natives.UnitAttackRestart | UnitAttackRestart}
         *             
         */
        public attackRestart(): boolean;
        /**
         *             Class member alias for {@link natives.UnitAttackFinish | UnitAttackFinish}
         *             
         */
        public attackFinish(): boolean;
        /**
         *             Class member alias for {@link natives.GetUnitAttackTypeByIndex | GetUnitAttackTypeByIndex}
         *             
         */
        public getAttackTypeByIndex(attackIndex: number): HandleHolder<"attacktype">;
        /**
         *             Class member alias for {@link natives.SetUnitAttackTypeByIndex | SetUnitAttackTypeByIndex}
         *             
         */
        public setAttackTypeByIndex(attackIndex: number, whichAttackType: HandleHolder<"attacktype">): void;
        /**
         *             Class member alias for {@link natives.GetUnitWeaponTypeByIndex | GetUnitWeaponTypeByIndex}
         *             
         */
        public getWeaponTypeByIndex(attackIndex: number): HandleHolder<"weapontype">;
        /**
         *             Class member alias for {@link natives.SetUnitWeaponTypeByIndex | SetUnitWeaponTypeByIndex}
         *             
         */
        public setWeaponTypeByIndex(attackIndex: number, whichWeaponType: HandleHolder<"weapontype">): void;
        /**
         *             Class member alias for {@link natives.GetUnitWeaponSoundByIndex | GetUnitWeaponSoundByIndex}
         *             
         */
        public getWeaponSoundByIndex(attackIndex: number): number;
        /**
         *             Class member alias for {@link natives.SetUnitWeaponSoundByIndex | SetUnitWeaponSoundByIndex}
         *             
         */
        public setWeaponSoundByIndex(attackIndex: number, weaponSound: number): void;
        /**
         *             Class member alias for {@link natives.GetUnitBaseDamageByIndex | GetUnitBaseDamageByIndex}
         *             
         */
        public getBaseDamageByIndex(attackIndex: number): number;
        /**
         *             Class member alias for {@link natives.SetUnitBaseDamageByIndex | SetUnitBaseDamageByIndex}
         *             
         */
        public setBaseDamageByIndex(attackIndex: number, damage: number): void;
        /**
         *             Class member alias for {@link natives.GetUnitBonusDamageByIndex | GetUnitBonusDamageByIndex}
         *             
         */
        public getBonusDamageByIndex(attackIndex: number): number;
        /**
         *             Class member alias for {@link natives.SetUnitBonusDamageByIndex | SetUnitBonusDamageByIndex}
         *             
         */
        public setBonusDamageByIndex(attackIndex: number, bonusDamage: number): void;
        /**
         *             Class member alias for {@link natives.GetUnitDamageDicesSideByIndex | GetUnitDamageDicesSideByIndex}
         *             
         */
        public getDamageDicesSideByIndex(attackIndex: number): number;
        /**
         *             Class member alias for {@link natives.SetUnitDamageDicesSideByIndex | SetUnitDamageDicesSideByIndex}
         *             
         */
        public setDamageDicesSideByIndex(attackIndex: number, dicesSides: number): void;
        /**
         *             Class member alias for {@link natives.GetUnitDamageDicesByIndex | GetUnitDamageDicesByIndex}
         *             
         */
        public getDamageDicesByIndex(attackIndex: number): number;
        /**
         *             Class member alias for {@link natives.SetUnitDamageDicesByIndex | SetUnitDamageDicesByIndex}
         *             
         */
        public setDamageDicesByIndex(attackIndex: number, dices: number): void;
        /**
         *             Class member alias for {@link natives.GetUnitMinimumDamageByIndex | GetUnitMinimumDamageByIndex}
         *             
         */
        public getMinimumDamageByIndex(attackIndex: number): number;
        /**
         *             Class member alias for {@link natives.GetUnitMaximumDamageByIndex | GetUnitMaximumDamageByIndex}
         *             
         */
        public getMaximumDamageByIndex(attackIndex: number): number;
        /**
         *             Class member alias for {@link natives.GetUnitAttackRangeByIndex | GetUnitAttackRangeByIndex}
         *             
         */
        public getAttackRangeByIndex(attackIndex: number): number;
        /**
         *             Class member alias for {@link natives.SetUnitAttackRangeByIndex | SetUnitAttackRangeByIndex}
         *             
         */
        public setAttackRangeByIndex(attackIndex: number, range: number): void;
        /**
         *             Class member alias for {@link natives.GetUnitAttackCooldownByIndex | GetUnitAttackCooldownByIndex}
         *             
         */
        public getAttackCooldownByIndex(attackIndex: number): number;
        /**
         *             Class member alias for {@link natives.SetUnitAttackCooldownByIndex | SetUnitAttackCooldownByIndex}
         *             
         */
        public setAttackCooldownByIndex(attackIndex: number, attackCooldown: number): void;
        /**
         *             Class member alias for {@link natives.GetUnitBackswingPointByIndex | GetUnitBackswingPointByIndex}
         *             
         */
        public getBackswingPointByIndex(attackIndex: number): number;
        /**
         *             Class member alias for {@link natives.SetUnitBackswingPointByIndex | SetUnitBackswingPointByIndex}
         *             
         */
        public setBackswingPointByIndex(attackIndex: number, backswing: number): void;
        /**
         *             Class member alias for {@link natives.GetUnitDamagePointByIndex | GetUnitDamagePointByIndex}
         *             
         */
        public getDamagePointByIndex(attackIndex: number): number;
        /**
         *             Class member alias for {@link natives.SetUnitDamagePointByIndex | SetUnitDamagePointByIndex}
         *             
         */
        public setDamagePointByIndex(attackIndex: number, damagePoint: number): void;
        /**
         *             Class member alias for {@link natives.GetUnitAttacksEnabledIndex | GetUnitAttacksEnabledIndex}
         *             
         */
        public getAttacksEnabledIndex(): number;
        /**
         *             Class member alias for {@link natives.GetUnitTotalMoveSpeed | GetUnitTotalMoveSpeed}
         *             
         */
        public getTotalMoveSpeed(): number;
        /**
         *             Class member alias for {@link natives.GetUnitVertexColour | GetUnitVertexColour}
         *             
         */
        public getVertexColour(): number;
        /**
         *             Class member alias for {@link natives.SetUnitModelEx | SetUnitModelEx}
         *             
         */
        public setModelEx(modelName: string, playercolourId: number): void;
        /**
         *             Class member alias for {@link natives.SetUnitMaterialTexture | SetUnitMaterialTexture}
         *             
         */
        public setMaterialTexture(textureName: string, materialId: number, textureIndex: number): void;
        /**
         *             Class member alias for {@link natives.SetUnitTexture | SetUnitTexture}
         *             
         */
        public setTexture(textureName: string, textureIndex: number): void;
        /**
         *             Class member alias for {@link natives.SetUnitReplaceableTexture | SetUnitReplaceableTexture}
         *             
         */
        public setReplaceableTexture(textureName: string, textureIndex: number): void;
        /**
         *             Class member alias for {@link natives.SetUnitFacingEx | SetUnitFacingEx}
         *             
         */
        public setFacingEx(facing: number, isInstant: boolean): void;
        /**
         *             Class member alias for {@link natives.GetUnitMinimapX | GetUnitMinimapX}
         *             
         */
        public getMinimapX(): number;
        /**
         *             Class member alias for {@link natives.GetUnitMinimapY | GetUnitMinimapY}
         *             
         */
        public getMinimapY(): number;
        /**
         *             Class member alias for {@link natives.GetUnitRallyPointX | GetUnitRallyPointX}
         *             
         */
        public getRallyPointX(): number;
        /**
         *             Class member alias for {@link natives.GetUnitRallyPointY | GetUnitRallyPointY}
         *             
         */
        public getRallyPointY(): number;
        /**
         *             Class member alias for {@link natives.GetHeroMaxLevelExperienceNeeded | GetHeroMaxLevelExperienceNeeded}
         *             
         */
        public getHeroMaxLevelExperienceNeeded(): number;
        /**
         *             Class member alias for {@link natives.GetHeroExperienceNeeded | GetHeroExperienceNeeded}
         *             
         */
        public getHeroExperienceNeeded(forLevel: number): number;
        /**
         *             Class member alias for {@link natives.UnitApplySilence | UnitApplySilence}
         *             
         */
        public applySilence(state: boolean): void;
        /**
         *             Class member alias for {@link natives.UnitDisableAbilities | UnitDisableAbilities}
         *             
         */
        public disableAbilities(state: boolean): void;
        /**
         *             Class member alias for {@link natives.PauseUnitEx | PauseUnitEx}
         *             
         */
        public pauseEx(pause: boolean): void;
        /**
         *             Class member alias for {@link natives.KillUnitEx | KillUnitEx}
         *             
         */
        public killEx(killer: Unit): void;
        /**
         *             Class member alias for {@link natives.GetUnitTarget | GetUnitTarget}
         *             
         */
        public getTarget(): Widget;
        /**
         *             Class member alias for {@link natives.GetUnitTargetUnit | GetUnitTargetUnit}
         *             
         */
        public getTargetUnit(): Unit;
        /**
         *             Class member alias for {@link natives.GetUnitTargetItem | GetUnitTargetItem}
         *             
         */
        public getTargetItem(): Item;
        /**
         *             Class member alias for {@link natives.GetUnitTargetDestructable | GetUnitTargetDestructable}
         *             
         */
        public getTargetDestructable(): Destructable;
        /**
         *             Class member alias for {@link natives.MorphUnitToTypeIdEx | MorphUnitToTypeIdEx}
         *             
         */
        public morphToTypeIdEx(uid: number, unitFlags: number, updateHealthState: boolean, updateManaState: boolean, healthStateId: number, manaStateId: number, updateScale: boolean, replaceAbilities: boolean, whichAbility: HandleHolder<"ability">, resetBuildingAnimation: boolean): void;
        /**
         *             Class member alias for {@link natives.MorphUnitToTypeId | MorphUnitToTypeId}
         *             
         */
        public morphToTypeId(uid: number): void;
        /**
         *             Class member alias for {@link natives.GetUnitModelObjectX | GetUnitModelObjectX}
         *             
         */
        public getModelObjectX(whichObject: string): number;
        /**
         *             Class member alias for {@link natives.GetUnitModelObjectY | GetUnitModelObjectY}
         *             
         */
        public getModelObjectY(whichObject: string): number;
        /**
         *             Class member alias for {@link natives.GetUnitModelObjectZ | GetUnitModelObjectZ}
         *             
         */
        public getModelObjectZ(whichObject: string): number;
        /**
         *             Class member alias for {@link natives.GetUnitModelObjectPositionLoc | GetUnitModelObjectPositionLoc}
         *             
         */
        public getModelObjectPositionLoc(whichObject: string): HLocation;
        /**
         *             Class member alias for {@link natives.GetUnitCurrentAnimationId | GetUnitCurrentAnimationId}
         *             
         */
        public getCurrentAnimationId(): number;
        /**
         *             Class member alias for {@link natives.GetUnitCurrentAnimationName | GetUnitCurrentAnimationName}
         *             
         */
        public getCurrentAnimationName(): string;
        /**
         *             Class member alias for {@link natives.IsUnitAutoOrientationEnabled | IsUnitAutoOrientationEnabled}
         *             
         */
        public isAutoOrientationEnabled(): boolean;
        /**
         *             Class member alias for {@link natives.UnitEnableAutoOrientation | UnitEnableAutoOrientation}
         *             
         */
        public enableAutoOrientation(enable: boolean): void;
        /**
         *             Class member alias for {@link natives.SetUnitMatrixScale | SetUnitMatrixScale}
         *             
         */
        public setMatrixScale(x: number, y: number, z: number): void;
        /**
         *             Class member alias for {@link natives.ResetUnitMatrix | ResetUnitMatrix}
         *             
         */
        public resetMatrix(): void;
        /**
         *             Class member alias for {@link natives.SetUnitOrientationEx | SetUnitOrientationEx}
         *             
         */
        public setOrientationEx(yaw: number, pitch: number, roll: number, eulerOrder: number): void;
        /**
         *             Class member alias for {@link natives.SetUnitOrientation | SetUnitOrientation}
         *             
         */
        public setOrientation(yaw: number, pitch: number, roll: number): void;
        /**
         *             Class member alias for {@link natives.UnitGetConstructionProgress | UnitGetConstructionProgress}
         *             
         */
        public getConstructionProgress(): number;
        /**
         *             Class member alias for {@link natives.UnitGetConstructionRemainingTime | UnitGetConstructionRemainingTime}
         *             
         */
        public getConstructionRemainingTime(): number;
        /**
         *             Class member alias for {@link natives.UnitSetConstructionRemainingTime | UnitSetConstructionRemainingTime}
         *             
         */
        public setConstructionRemainingTime(time: number): void;
        /**
         *             Class member alias for {@link natives.UnitGetUpgradeProgress | UnitGetUpgradeProgress}
         *             
         */
        public getUpgradeProgress(): number;
        /**
         *             Class member alias for {@link natives.UnitGetUpgradeRemainingTime | UnitGetUpgradeRemainingTime}
         *             
         */
        public getUpgradeRemainingTime(): number;
        /**
         *             Class member alias for {@link natives.UnitSetUpgradeRemainingTime | UnitSetUpgradeRemainingTime}
         *             
         */
        public setUpgradeRemainingTime(time: number): void;
        /**
         *             Class member alias for {@link natives.CreateIllusionFromUnit | CreateIllusionFromUnit}
         *             
         */
        public createIllusionFrom(): Unit;
        /**
         *             Class member alias for {@link natives.CreateIllusionFromUnitEx | CreateIllusionFromUnitEx}
         *             
         */
        public createIllusionFromEx(copyPassives: boolean): Unit;
        /**
         *             Class member alias for {@link natives.GetIllusionDamageDealt | GetIllusionDamageDealt}
         *             
         */
        public getIllusionDamageDealt(): number;
        /**
         *             Class member alias for {@link natives.SetIllusionDamageDealt | SetIllusionDamageDealt}
         *             
         */
        public setIllusionDamageDealt(multiplier: number): void;
        /**
         *             Class member alias for {@link natives.GetIllusionDamageReceived | GetIllusionDamageReceived}
         *             
         */
        public getIllusionDamageReceived(): number;
        /**
         *             Class member alias for {@link natives.SetIllusionDamageReceived | SetIllusionDamageReceived}
         *             
         */
        public setIllusionDamageReceived(multiplier: number): void;
        /**
         *             Class member alias for {@link natives.QueueImmediateOrderById | QueueImmediateOrderById}
         *             
         */
        public queueImmediateOrderById(order: number): boolean;
        /**
         *             Class member alias for {@link natives.QueuePointOrderById | QueuePointOrderById}
         *             
         */
        public queuePointOrderById(order: number, x: number, y: number): boolean;
        /**
         *             Class member alias for {@link natives.QueueTargetOrderById | QueueTargetOrderById}
         *             
         */
        public queueTargetOrderById(order: number, targetWidget: Widget): boolean;
        /**
         *             Class member alias for {@link natives.QueueInstantPointOrderById | QueueInstantPointOrderById}
         *             
         */
        public queueInstantPointOrderById(order: number, x: number, y: number, instantTargetWidget: Widget): boolean;
        /**
         *             Class member alias for {@link natives.QueueInstantTargetOrderById | QueueInstantTargetOrderById}
         *             
         */
        public queueInstantTargetOrderById(order: number, targetWidget: Widget, instantTargetWidget: Widget): boolean;
        /**
         *             Class member alias for {@link natives.QueueBuildOrderById | QueueBuildOrderById}
         *             
         */
        public queueBuildOrderById(unitId: number, x: number, y: number): boolean;
        /**
         *             Class member alias for {@link natives.GetUnitOrderCount | GetUnitOrderCount}
         *             
         */
        public getOrderCount(): number;
        /**
         *             Class member alias for {@link natives.GetUnitOrderIdByIndex | GetUnitOrderIdByIndex}
         *             
         */
        public getOrderIdByIndex(index: number): number;
        /**
         *             Class member alias for {@link natives.UnitClearOrders | UnitClearOrders}
         *             
         */
        public clearOrders(onlyQueued: boolean): void;
        /**
         *             Class member alias for {@link natives.UnitForceStopOrder | UnitForceStopOrder}
         *             
         */
        public forceStopOrder(clearQueue: boolean): void;
        /**
         *             Class member alias for {@link natives.CreateProjectileEx | CreateProjectileEx}
         *             
         */
        public createProjectileEx(projectileTypeId: number, attackIndex: number): HandleHolder<"projectile">;
        /**
         *             Class member alias for {@link natives.CreateMinimapIconOnUnit | CreateMinimapIconOnUnit}
         *             
         */
        public createMinimapIconOn(red: number, green: number, blue: number, pingPath: string, fogVisibility: HandleHolder<"fogstate">): HandleHolder<"minimapicon">;
        /**
         *             Class member alias for {@link natives.BlzGetUnitMaxHP | BlzGetUnitMaxHP}
         *             
         */
        public blzGetMaxHP(): number;
        /**
         *             Class member alias for {@link natives.BlzSetUnitMaxHP | BlzSetUnitMaxHP}
         *             
         */
        public blzSetMaxHP(hp: number): void;
        /**
         *             Class member alias for {@link natives.BlzGetUnitMaxMana | BlzGetUnitMaxMana}
         *             
         */
        public blzGetMaxMana(): number;
        /**
         *             Class member alias for {@link natives.BlzSetUnitMaxMana | BlzSetUnitMaxMana}
         *             
         */
        public blzSetMaxMana(mana: number): void;
        /**
         *             Class member alias for {@link natives.BlzSetUnitName | BlzSetUnitName}
         *             
         */
        public blzSetName(name: string): void;
        /**
         *             Class member alias for {@link natives.BlzSetHeroProperName | BlzSetHeroProperName}
         *             
         */
        public blzSetHeroProperName(heroProperName: string): void;
        /**
         *             Class member alias for {@link natives.BlzGetUnitBaseDamage | BlzGetUnitBaseDamage}
         *             
         */
        public blzGetBaseDamage(weaponIndex: number): number;
        /**
         *             Class member alias for {@link natives.BlzSetUnitBaseDamage | BlzSetUnitBaseDamage}
         *             
         */
        public blzSetBaseDamage(baseDamage: number, weaponIndex: number): void;
        /**
         *             Class member alias for {@link natives.BlzGetUnitDiceNumber | BlzGetUnitDiceNumber}
         *             
         */
        public blzGetDiceNumber(weaponIndex: number): number;
        /**
         *             Class member alias for {@link natives.BlzSetUnitDiceNumber | BlzSetUnitDiceNumber}
         *             
         */
        public blzSetDiceNumber(diceNumber: number, weaponIndex: number): void;
        /**
         *             Class member alias for {@link natives.BlzGetUnitDiceSides | BlzGetUnitDiceSides}
         *             
         */
        public blzGetDiceSides(weaponIndex: number): number;
        /**
         *             Class member alias for {@link natives.BlzSetUnitDiceSides | BlzSetUnitDiceSides}
         *             
         */
        public blzSetDiceSides(diceSides: number, weaponIndex: number): void;
        /**
         *             Class member alias for {@link natives.BlzGetUnitAttackCooldown | BlzGetUnitAttackCooldown}
         *             
         */
        public blzGetAttackCooldown(weaponIndex: number): number;
        /**
         *             Class member alias for {@link natives.BlzSetUnitAttackCooldown | BlzSetUnitAttackCooldown}
         *             
         */
        public blzSetAttackCooldown(cooldown: number, weaponIndex: number): void;
        /**
         *             Class member alias for {@link natives.BlzGetUnitArmor | BlzGetUnitArmor}
         *             
         */
        public blzGetArmor(): number;
        /**
         *             Class member alias for {@link natives.BlzSetUnitArmor | BlzSetUnitArmor}
         *             
         */
        public blzSetArmor(armorAmount: number): void;
        /**
         *             Class member alias for {@link natives.BlzUnitHideAbility | BlzUnitHideAbility}
         *             
         */
        public blzHideAbility(abilId: number, flag: boolean): void;
        /**
         *             Class member alias for {@link natives.BlzUnitDisableAbility | BlzUnitDisableAbility}
         *             
         */
        public blzDisableAbility(abilId: number, flag: boolean, hideUI: boolean): void;
        /**
         *             Class member alias for {@link natives.BlzUnitCancelTimedLife | BlzUnitCancelTimedLife}
         *             
         */
        public blzCancelTimedLife(): void;
        /**
         *             Class member alias for {@link natives.BlzIsUnitSelectable | BlzIsUnitSelectable}
         *             
         */
        public blzIsSelectable(): boolean;
        /**
         *             Class member alias for {@link natives.BlzIsUnitInvulnerable | BlzIsUnitInvulnerable}
         *             
         */
        public blzIsInvulnerable(): boolean;
        /**
         *             Class member alias for {@link natives.BlzUnitInterruptAttack | BlzUnitInterruptAttack}
         *             
         */
        public blzInterruptAttack(): void;
        /**
         *             Class member alias for {@link natives.BlzGetUnitCollisionSize | BlzGetUnitCollisionSize}
         *             
         */
        public blzGetCollisionSize(): number;
        /**
         *             Class member alias for {@link natives.BlzSetUnitAbilityCooldown | BlzSetUnitAbilityCooldown}
         *             
         */
        public blzSetAbilityCooldown(abilId: number, level: number, cooldown: number): void;
        /**
         *             Class member alias for {@link natives.BlzGetUnitAbilityCooldown | BlzGetUnitAbilityCooldown}
         *             
         */
        public blzGetAbilityCooldown(abilId: number, level: number): number;
        /**
         *             Class member alias for {@link natives.BlzGetUnitAbilityCooldownRemaining | BlzGetUnitAbilityCooldownRemaining}
         *             
         */
        public blzGetAbilityCooldownRemaining(abilId: number): number;
        /**
         *             Class member alias for {@link natives.BlzEndUnitAbilityCooldown | BlzEndUnitAbilityCooldown}
         *             
         */
        public blzEndAbilityCooldown(abilCode: number): void;
        /**
         *             Class member alias for {@link natives.BlzStartUnitAbilityCooldown | BlzStartUnitAbilityCooldown}
         *             
         */
        public blzStartAbilityCooldown(abilCode: number, cooldown: number): void;
        /**
         *             Class member alias for {@link natives.BlzGetUnitAbilityManaCost | BlzGetUnitAbilityManaCost}
         *             
         */
        public blzGetAbilityManaCost(abilId: number, level: number): number;
        /**
         *             Class member alias for {@link natives.BlzSetUnitAbilityManaCost | BlzSetUnitAbilityManaCost}
         *             
         */
        public blzSetAbilityManaCost(abilId: number, level: number, manaCost: number): void;
        /**
         *             Class member alias for {@link natives.BlzGetLocalUnitZ | BlzGetLocalUnitZ}
         *             
         */
        public blzGetLocalZ(): number;
        /**
         *             Class member alias for {@link natives.BlzGetUnitZ | BlzGetUnitZ}
         *             
         */
        public blzGetZ(): number;
        /**
         *             Class member alias for {@link natives.BlzShowUnitTeamGlow | BlzShowUnitTeamGlow}
         *             
         */
        public blzShowTeamGlow(show: boolean): void;
        /**
         *             Class member alias for {@link natives.BlzGetUnitAbility | BlzGetUnitAbility}
         *             
         */
        public blzGetAbility(abilId: number): HandleHolder<"ability">;
        /**
         *             Class member alias for {@link natives.BlzGetUnitAbilityByIndex | BlzGetUnitAbilityByIndex}
         *             
         */
        public blzGetAbilityByIndex(index: number): HandleHolder<"ability">;
        /**
         *             Class member alias for {@link natives.BlzPauseUnitEx | BlzPauseUnitEx}
         *             
         */
        public blzPauseEx(flag: boolean): void;
        /**
         *             Class member alias for {@link natives.BlzSetUnitFacingEx | BlzSetUnitFacingEx}
         *             
         */
        public blzSetFacingEx(facingAngle: number): void;
        /**
         *             Class member alias for {@link natives.BlzGetUnitBooleanField | BlzGetUnitBooleanField}
         *             
         */
        public blzGetBooleanField(whichField: HandleHolder<"unitbooleanfield">): boolean;
        /**
         *             Class member alias for {@link natives.BlzGetUnitIntegerField | BlzGetUnitIntegerField}
         *             
         */
        public blzGetIntegerField(whichField: HandleHolder<"unitintegerfield">): number;
        /**
         *             Class member alias for {@link natives.BlzGetUnitRealField | BlzGetUnitRealField}
         *             
         */
        public blzGetRealField(whichField: HandleHolder<"unitrealfield">): number;
        /**
         *             Class member alias for {@link natives.BlzGetUnitStringField | BlzGetUnitStringField}
         *             
         */
        public blzGetStringField(whichField: HandleHolder<"unitstringfield">): string;
        /**
         *             Class member alias for {@link natives.BlzSetUnitBooleanField | BlzSetUnitBooleanField}
         *             
         */
        public blzSetBooleanField(whichField: HandleHolder<"unitbooleanfield">, value: boolean): boolean;
        /**
         *             Class member alias for {@link natives.BlzSetUnitIntegerField | BlzSetUnitIntegerField}
         *             
         */
        public blzSetIntegerField(whichField: HandleHolder<"unitintegerfield">, value: number): boolean;
        /**
         *             Class member alias for {@link natives.BlzSetUnitRealField | BlzSetUnitRealField}
         *             
         */
        public blzSetRealField(whichField: HandleHolder<"unitrealfield">, value: number): boolean;
        /**
         *             Class member alias for {@link natives.BlzSetUnitStringField | BlzSetUnitStringField}
         *             
         */
        public blzSetStringField(whichField: HandleHolder<"unitstringfield">, value: string): boolean;
        /**
         *             Class member alias for {@link natives.BlzGetUnitWeaponBooleanField | BlzGetUnitWeaponBooleanField}
         *             
         */
        public blzGetWeaponBooleanField(whichField: HandleHolder<"unitweaponbooleanfield">, index: number): boolean;
        /**
         *             Class member alias for {@link natives.BlzGetUnitWeaponIntegerField | BlzGetUnitWeaponIntegerField}
         *             
         */
        public blzGetWeaponIntegerField(whichField: HandleHolder<"unitweaponintegerfield">, index: number): number;
        /**
         *             Class member alias for {@link natives.BlzGetUnitWeaponRealField | BlzGetUnitWeaponRealField}
         *             
         */
        public blzGetWeaponRealField(whichField: HandleHolder<"unitweaponrealfield">, index: number): number;
        /**
         *             Class member alias for {@link natives.BlzGetUnitWeaponStringField | BlzGetUnitWeaponStringField}
         *             
         */
        public blzGetWeaponStringField(whichField: HandleHolder<"unitweaponstringfield">, index: number): string;
        /**
         *             Class member alias for {@link natives.BlzSetUnitWeaponBooleanField | BlzSetUnitWeaponBooleanField}
         *             
         */
        public blzSetWeaponBooleanField(whichField: HandleHolder<"unitweaponbooleanfield">, index: number, value: boolean): boolean;
        /**
         *             Class member alias for {@link natives.BlzSetUnitWeaponIntegerField | BlzSetUnitWeaponIntegerField}
         *             
         */
        public blzSetWeaponIntegerField(whichField: HandleHolder<"unitweaponintegerfield">, index: number, value: number): boolean;
        /**
         *             Class member alias for {@link natives.BlzSetUnitWeaponRealField | BlzSetUnitWeaponRealField}
         *             
         */
        public blzSetWeaponRealField(whichField: HandleHolder<"unitweaponrealfield">, index: number, value: number): boolean;
        /**
         *             Class member alias for {@link natives.BlzSetUnitWeaponStringField | BlzSetUnitWeaponStringField}
         *             
         */
        public blzSetWeaponStringField(whichField: HandleHolder<"unitweaponstringfield">, index: number, value: string): boolean;
        /**
         *             Class member alias for {@link natives.BlzGetUnitSkin | BlzGetUnitSkin}
         *             
         */
        public blzGetSkin(): number;
        /**
         *             Class member alias for {@link natives.BlzSetUnitSkin | BlzSetUnitSkin}
         *             
         */
        public blzSetSkin(skinId: number): void;
        /**
         *             Class member alias for {@link natives.BlzQueueImmediateOrderById | BlzQueueImmediateOrderById}
         *             
         */
        public blzQueueImmediateOrderById(order: number): boolean;
        /**
         *             Class member alias for {@link natives.BlzQueuePointOrderById | BlzQueuePointOrderById}
         *             
         */
        public blzQueuePointOrderById(order: number, x: number, y: number): boolean;
        /**
         *             Class member alias for {@link natives.BlzQueueTargetOrderById | BlzQueueTargetOrderById}
         *             
         */
        public blzQueueTargetOrderById(order: number, targetWidget: Widget): boolean;
        /**
         *             Class member alias for {@link natives.BlzQueueInstantPointOrderById | BlzQueueInstantPointOrderById}
         *             
         */
        public blzQueueInstantPointOrderById(order: number, x: number, y: number, instantTargetWidget: Widget): boolean;
        /**
         *             Class member alias for {@link natives.BlzQueueInstantTargetOrderById | BlzQueueInstantTargetOrderById}
         *             
         */
        public blzQueueInstantTargetOrderById(order: number, targetWidget: Widget, instantTargetWidget: Widget): boolean;
        /**
         *             Class member alias for {@link natives.BlzQueueBuildOrderById | BlzQueueBuildOrderById}
         *             
         */
        public blzQueueBuildOrderById(unitId: number, x: number, y: number): boolean;
        /**
         *             Class member alias for {@link natives.BlzGetUnitOrderCount | BlzGetUnitOrderCount}
         *             
         */
        public blzGetOrderCount(): number;
        /**
         *             Class member alias for {@link natives.BlzUnitClearOrders | BlzUnitClearOrders}
         *             
         */
        public blzClearOrders(onlyQueued: boolean): void;
        /**
         *             Class member alias for {@link natives.BlzUnitForceStopOrder | BlzUnitForceStopOrder}
         *             
         */
        public blzForceStopOrder(clearQueue: boolean): void;
    }

    export type UnitOptions = {
        	type: "building";
        	isAutoBuild?: boolean;
        	workersCanAssist?: boolean;
        } | {
        	type: "illusion";
        } | {
        	type: "corpse";
        };
    export type UnitIllusionOptions = {
        	copyPassives: boolean;
        };

    const isUjApi: boolean;
    const console: Console;
}
export {};
